# -*- coding: utf-8 -*-

from numpy import transpose


def edit_matrix(self, matrix):
    """To apply transformation on the matrix

    Parameters
    ----------
    self : ImportMatrix
        An ImportMatrix object
    matrix: ndarray
        The matrix to edit

    Returns
    -------
    matrix: ndarray
        The edited matrix

    """

    if self.is_transpose:
        return transpose(matrix)
    return matrix



# -*- coding: utf-8 -*-


def get_begin(self):
    """Return the begin point of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object

    Returns
    -------
    begin: complex
        begin point of the Arc2
    """

    return self.begin



# -*- coding: utf-8 -*-
from .Save.save_json import save_json
from .Save.save_hdf5 import save_hdf5
from .Save.save_pkl import save_pkl


class FormatError(Exception):
    pass


def save(self, save_path="", is_folder=False):
    """Save the object to the save_path

    Parameters
    ----------
    self :
        A pyleecan object
    save_path: str
        path to the folder to save the object
    is_folder: bool
        to split the object in different files: separate simulation machine and materials
    """
    # Save in the object.path if it exist and save_path is empty
    if save_path == "" and hasattr(self, "path") and getattr(self, "path") != None:
        save_path = self.path
    if (
        not save_path.endswith("json")
        and not save_path.endswith("h5")
        and not save_path.endswith("pkl")
        and not is_folder
    ):
        save_path += ".json"  # Default format

    # Avoid errors like "\t" in path
    save_path = save_path.replace("\\", "/")
    # Save in json
    if save_path.endswith(".json") or is_folder:
        save_json(self, save_path=save_path, is_folder=is_folder)
    # Save in hdf5
    elif save_path.endswith(".h5"):
        save_hdf5(self, save_path=save_path)
    # Save in pkl
    elif save_path.endswith(".pkl"):
        save_pkl(self, save_path=save_path)
    else:
        raise FormatError(
            "Unknown file extension: '{}'".format(save_path.rsplit(".")[-1])
        )



def pop(self, i):
    """
    Remove the item at the given position in the list, and return it.
    If no index is specified, a.pop() removes and returns the last item in the list.
    """
    return self.list_output.pop(i)



from ....Methods.Output.XOutput import XOutputError


def get_paramexplorer(self, symbol):
    """Get the ParamExplorer with symbol
    Parameters
    ----------
    symbol: str
        ParamExplorer symbol
    """
    for paramexplorer in self.paramexplorer_list:
        if paramexplorer.symbol == symbol:
            return paramexplorer

    raise XOutputError("Unknown symbol {}".format(symbol))



# -*- coding: utf-8 -*-
from numpy import mean, max as np_max, min as np_min

from SciDataTool import DataTime, VectorField, Data1D

from ....Functions.Winding.gen_phase_list import gen_name


def store(self, out_dict, axes_dict):
    """Store the standard outputs of Magnetics that are temporarily in out_dict as arrays into OutMag as Data object

    Parameters
    ----------
    self : OutMag
        the OutMag object to update
    out_dict : dict
        Dict containing all magnetic quantities that have been calculated in comp_flux_airgap
    axes_dict: {Data}
        Dict of axes used for magnetic calculation

    """

    # Get time axis
    Time = axes_dict["Time"]

    # Store airgap flux as VectorField object
    # Axes for each airgap flux component
    axis_list = [Time, axes_dict["Angle"]]
    # Create VectorField with empty components
    self.B = VectorField(
        name="Airgap flux density",
        symbol="B",
    )
    # Radial flux component
    if "Br" in out_dict:
        self.B.components["radial"] = DataTime(
            name="Airgap radial flux density",
            unit="T",
            symbol="B_r",
            axes=axis_list,
            values=out_dict.pop("Br"),
        )
    # Tangential flux component
    if "Bt" in out_dict:
        self.B.components["tangential"] = DataTime(
            name="Airgap tangential flux density",
            unit="T",
            symbol="B_t",
            axes=axis_list,
            values=out_dict.pop("Bt"),
        )
    # Axial flux component
    if "Bz" in out_dict:
        self.B.components["axial"] = DataTime(
            name="Airgap axial flux density",
            unit="T",
            symbol="B_z",
            axes=axis_list,
            values=out_dict.pop("Bz"),
        )

    # Store electromagnetic torque over time, and global values: average, peak to peak and ripple
    if "Tem" in out_dict:

        Tem = out_dict.pop("Tem")

        self.Tem = DataTime(
            name="Electromagnetic torque",
            unit="Nm",
            symbol="T_{em}",
            axes=[axes_dict["Time_Tem"]],
            values=Tem,
        )

        # Calculate average torque in Nm
        self.Tem_av = mean(Tem)
        self.get_logger().debug("Average Torque: " + str(self.Tem_av) + " N.m")

        # Calculate peak to peak torque in absolute value Nm
        self.Tem_rip_pp = abs(np_max(Tem) - np_min(Tem))  # [N.m]

        # Calculate torque ripple in percentage
        if self.Tem_av != 0:
            self.Tem_rip_norm = self.Tem_rip_pp / self.Tem_av  # []
        else:
            self.Tem_rip_norm = None

    # Store list of winding fluxlinkage, stator winding fluxlinkage
    # and calculate electromotive force
    if "Phi_wind" in out_dict:
        machine = self.parent.simu.machine
        self.Phi_wind = {}
        for key in out_dict["Phi_wind"].keys():
            # Store stator winding flux
            lam = machine.get_lam_by_label(key)
            qs = lam.winding.qs

            Phase = Data1D(
                name="phase",
                unit="",
                values=gen_name(qs),
                is_components=True,
            )
            prefix = "Stator" if lam.is_stator else "Rotor"
            self.Phi_wind[key] = DataTime(
                name=prefix + " Winding Flux",
                unit="Wb",
                symbol="Phi_{wind}",
                axes=[Time, Phase],
                values=out_dict["Phi_wind"][key],
            )

        if "Stator_0" in out_dict["Phi_wind"].keys():  # TODO fix for multi stator
            self.Phi_wind_stator = self.Phi_wind["Stator_0"]

        # Electromotive force computation
        self.comp_emf()

        # remove from out_dict
        out_dict.pop("Phi_wind")

    # Store MeshSolution object
    if "meshsolution" in out_dict:
        self.meshsolution = out_dict.pop("meshsolution")

    if "Rag" in out_dict:
        self.Rag = out_dict.pop("Rag")



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW22 import *


def check(self):
    """Check that the SlotW22 object is correct

    Parameters
    ----------
    self : SlotW22
        A SlotW22 object

    Returns
    -------
    None

    Raises
    -------
    S22_A02CheckError
        You must have W0 <= W2
    S22_SpCheckError
        You must have W2 < 2*pi/Zs
    """
    if self.W2 < self.W0:
        raise S22_A02CheckError("You must have W0 <= W2")

    if 2 * pi / self.Zs <= self.W2:
        raise S22_SpCheckError("You must have W2 < 2*pi/Zs")



# -*- coding: utf-8 -*-
from numpy import pi


def comp_surface_active(self):
    """Compute the Slot inner active surface (by analytical computation)

    Parameters
    ----------
    self : SlotM17
        A SlotM17 object

    Returns
    -------
    Swind: float
        Slot inner active surface [m**2]

    """

    return pi * self.parent.Rext ** 2 / 2



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Mode.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Mode
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .SolutionMat import SolutionMat

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.Mode.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Simulation.Mode.plot_animated import plot_animated
except ImportError as error:
    plot_animated = error

try:
    from ..Methods.Simulation.Mode.get_shape_xyz import get_shape_xyz
except ImportError as error:
    get_shape_xyz = error

try:
    from ..Methods.Simulation.Mode.get_shape_pol import get_shape_pol
except ImportError as error:
    get_shape_pol = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError


class Mode(SolutionMat):
    """Structural module: Mode object"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.Mode.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Mode method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Simulation.Mode.plot_animated
    if isinstance(plot_animated, ImportError):
        plot_animated = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Mode method plot_animated: " + str(plot_animated)
                )
            )
        )
    else:
        plot_animated = plot_animated
    # cf Methods.Simulation.Mode.get_shape_xyz
    if isinstance(get_shape_xyz, ImportError):
        get_shape_xyz = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Mode method get_shape_xyz: " + str(get_shape_xyz)
                )
            )
        )
    else:
        get_shape_xyz = get_shape_xyz
    # cf Methods.Simulation.Mode.get_shape_pol
    if isinstance(get_shape_pol, ImportError):
        get_shape_pol = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Mode method get_shape_pol: " + str(get_shape_pol)
                )
            )
        )
    else:
        get_shape_pol = get_shape_pol
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        nat_freq=None,
        order_circ=None,
        order_long=None,
        field=None,
        indice=None,
        axis_name=None,
        axis_size=None,
        type_cell="triangle",
        label=None,
        dimension=2,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "nat_freq" in list(init_dict.keys()):
                nat_freq = init_dict["nat_freq"]
            if "order_circ" in list(init_dict.keys()):
                order_circ = init_dict["order_circ"]
            if "order_long" in list(init_dict.keys()):
                order_long = init_dict["order_long"]
            if "field" in list(init_dict.keys()):
                field = init_dict["field"]
            if "indice" in list(init_dict.keys()):
                indice = init_dict["indice"]
            if "axis_name" in list(init_dict.keys()):
                axis_name = init_dict["axis_name"]
            if "axis_size" in list(init_dict.keys()):
                axis_size = init_dict["axis_size"]
            if "type_cell" in list(init_dict.keys()):
                type_cell = init_dict["type_cell"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "dimension" in list(init_dict.keys()):
                dimension = init_dict["dimension"]
        # Set the properties (value check and convertion are done in setter)
        self.nat_freq = nat_freq
        self.order_circ = order_circ
        self.order_long = order_long
        # Call SolutionMat init
        super(Mode, self).__init__(
            field=field,
            indice=indice,
            axis_name=axis_name,
            axis_size=axis_size,
            type_cell=type_cell,
            label=label,
            dimension=dimension,
        )
        # The class is frozen (in SolutionMat init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Mode_str = ""
        # Get the properties inherited from SolutionMat
        Mode_str += super(Mode, self).__str__()
        Mode_str += "nat_freq = " + str(self.nat_freq) + linesep
        Mode_str += "order_circ = " + str(self.order_circ) + linesep
        Mode_str += "order_long = " + str(self.order_long) + linesep
        return Mode_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from SolutionMat
        if not super(Mode, self).__eq__(other):
            return False
        if other.nat_freq != self.nat_freq:
            return False
        if other.order_circ != self.order_circ:
            return False
        if other.order_long != self.order_long:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from SolutionMat
        diff_list.extend(super(Mode, self).compare(other, name=name))
        if other._nat_freq != self._nat_freq:
            diff_list.append(name + ".nat_freq")
        if other._order_circ != self._order_circ:
            diff_list.append(name + ".order_circ")
        if other._order_long != self._order_long:
            diff_list.append(name + ".order_long")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from SolutionMat
        S += super(Mode, self).__sizeof__()
        S += getsizeof(self.nat_freq)
        S += getsizeof(self.order_circ)
        S += getsizeof(self.order_long)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from SolutionMat
        Mode_dict = super(Mode, self).as_dict(**kwargs)
        Mode_dict["nat_freq"] = self.nat_freq
        Mode_dict["order_circ"] = self.order_circ
        Mode_dict["order_long"] = self.order_long
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        Mode_dict["__class__"] = "Mode"
        return Mode_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.nat_freq = None
        self.order_circ = None
        self.order_long = None
        # Set to None the properties inherited from SolutionMat
        super(Mode, self)._set_None()

    def _get_nat_freq(self):
        """getter of nat_freq"""
        return self._nat_freq

    def _set_nat_freq(self, value):
        """setter of nat_freq"""
        check_var("nat_freq", value, "float")
        self._nat_freq = value

    nat_freq = property(
        fget=_get_nat_freq,
        fset=_set_nat_freq,
        doc=u"""Natural frequency of the mode

        :Type: float
        """,
    )

    def _get_order_circ(self):
        """getter of order_circ"""
        return self._order_circ

    def _set_order_circ(self, value):
        """setter of order_circ"""
        check_var("order_circ", value, "int", Vmin=0)
        self._order_circ = value

    order_circ = property(
        fget=_get_order_circ,
        fset=_set_order_circ,
        doc=u"""Circumferential order

        :Type: int
        :min: 0
        """,
    )

    def _get_order_long(self):
        """getter of order_long"""
        return self._order_long

    def _set_order_long(self, value):
        """setter of order_long"""
        check_var("order_long", value, "int", Vmin=0)
        self._order_long = value

    order_long = property(
        fget=_get_order_long,
        fset=_set_order_long,
        doc=u"""Longitudinal order

        :Type: int
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/Conductor.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/Conductor
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.Conductor.check import check
except ImportError as error:
    check = error


from ._check import InitUnKnowClassError
from .Material import Material


class Conductor(FrozenClass):
    """abstact class for conductors"""

    VERSION = 1

    # cf Methods.Machine.Conductor.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use Conductor method check: " + str(check))
            )
        )
    else:
        check = check
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, cond_mat=-1, ins_mat=-1, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "cond_mat" in list(init_dict.keys()):
                cond_mat = init_dict["cond_mat"]
            if "ins_mat" in list(init_dict.keys()):
                ins_mat = init_dict["ins_mat"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.cond_mat = cond_mat
        self.ins_mat = ins_mat

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Conductor_str = ""
        if self.parent is None:
            Conductor_str += "parent = None " + linesep
        else:
            Conductor_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.cond_mat is not None:
            tmp = self.cond_mat.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Conductor_str += "cond_mat = " + tmp
        else:
            Conductor_str += "cond_mat = None" + linesep + linesep
        if self.ins_mat is not None:
            tmp = self.ins_mat.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Conductor_str += "ins_mat = " + tmp
        else:
            Conductor_str += "ins_mat = None" + linesep + linesep
        return Conductor_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.cond_mat != self.cond_mat:
            return False
        if other.ins_mat != self.ins_mat:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.cond_mat is None and self.cond_mat is not None) or (
            other.cond_mat is not None and self.cond_mat is None
        ):
            diff_list.append(name + ".cond_mat None mismatch")
        elif self.cond_mat is not None:
            diff_list.extend(
                self.cond_mat.compare(other.cond_mat, name=name + ".cond_mat")
            )
        if (other.ins_mat is None and self.ins_mat is not None) or (
            other.ins_mat is not None and self.ins_mat is None
        ):
            diff_list.append(name + ".ins_mat None mismatch")
        elif self.ins_mat is not None:
            diff_list.extend(
                self.ins_mat.compare(other.ins_mat, name=name + ".ins_mat")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.cond_mat)
        S += getsizeof(self.ins_mat)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Conductor_dict = dict()
        if self.cond_mat is None:
            Conductor_dict["cond_mat"] = None
        else:
            Conductor_dict["cond_mat"] = self.cond_mat.as_dict(**kwargs)
        if self.ins_mat is None:
            Conductor_dict["ins_mat"] = None
        else:
            Conductor_dict["ins_mat"] = self.ins_mat.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        Conductor_dict["__class__"] = "Conductor"
        return Conductor_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.cond_mat is not None:
            self.cond_mat._set_None()
        if self.ins_mat is not None:
            self.ins_mat._set_None()

    def _get_cond_mat(self):
        """getter of cond_mat"""
        return self._cond_mat

    def _set_cond_mat(self, value):
        """setter of cond_mat"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "cond_mat"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Material()
        check_var("cond_mat", value, "Material")
        self._cond_mat = value

        if self._cond_mat is not None:
            self._cond_mat.parent = self

    cond_mat = property(
        fget=_get_cond_mat,
        fset=_set_cond_mat,
        doc=u"""Material of the conductor

        :Type: Material
        """,
    )

    def _get_ins_mat(self):
        """getter of ins_mat"""
        return self._ins_mat

    def _set_ins_mat(self, value):
        """setter of ins_mat"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "ins_mat"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Material()
        check_var("ins_mat", value, "Material")
        self._ins_mat = value

        if self._ins_mat is not None:
            self._ins_mat.parent = self

    ins_mat = property(
        fget=_get_ins_mat,
        fset=_set_ins_mat,
        doc=u"""Material of the insulation

        :Type: Material
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi, arcsin, exp, angle


def get_angle(self, is_deg=False):
    """Return the angle of the arc

    Parameters
    ----------
    self : Arc1
        An Arc1 object
    is_deg: bool
        True to convert to degree

    Returns
    -------
    angle: float
        Angle of the arc
    """
    # Go to center ref with begin on the X > 0 axis
    Zc = self.get_center()
    Z2 = (self.end - Zc) * exp(-1j * angle(self.begin - Zc))

    if Z2.imag > 0 and self.is_trigo_direction:
        alpha = angle(Z2)
    elif Z2.imag > 0 and not self.is_trigo_direction:
        alpha = -(2 * pi - angle(Z2))
    elif Z2.imag < 0 and self.is_trigo_direction:
        alpha = 2 * pi - abs(angle(Z2))
    elif Z2.imag < 0 and not self.is_trigo_direction:
        alpha = angle(Z2)
    elif Z2.imag == 0 and self.is_trigo_direction:
        alpha = abs(angle(Z2))
    elif Z2.imag == 0 and not self.is_trigo_direction:
        alpha = -abs(angle(Z2))
    else:
        alpha = 0

    if is_deg:
        return alpha * 180 / pi
    else:
        return alpha



# -*- coding: utf-8 -*-
from numpy import pi


def comp_surface(self):
    """Compute the circle surface

    Parameters
    ----------
    self : Circle
        A Circle object

    Returns
    -------
    surf: float
        The circle surface [m**2]

    """

    return pi * self.radius ** 2



# -*- coding: utf-8 -*-
from numpy import pi, all as np_all

from ...Functions.FEMM.set_FEMM_circuit_prop import set_FEMM_circuit_prop


def update_FEMM_simulation(
    femm,
    circuits,
    is_internal_rotor,
    is_sliding_band,
    angle_rotor,
    Is,
    Ir,
    ii,
):
    """Update the simulation by changing the rotor position and
    updating the currents


    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    circuits :
        Output object
    is_internal_rotor: bool
        True if it is an internal rotor topology
    is_sliding_band: bool
        True if it is an internal rotor topology
    angle_rotor: ndarray
        Vector of rotor angular position over time [Nt,]
    Is : ndarray
        Stator currents function of phase and time [qs,Nt]
    Ir : ndarray
        Rotor currents function of phase and time [qr,Nt]
    ii : int
        Time step index

    """

    if is_sliding_band:  # No rotation without sliding band.
        # Rotor rotation using sliding band
        if is_internal_rotor:
            femm.mi_modifyboundprop("bc_ag2", 10, 180 * angle_rotor[ii] / pi)
        else:
            femm.mi_modifyboundprop("bc_ag2", 11, 180 * angle_rotor[ii] / pi)

    # Update currents
    for label in circuits:
        if "Circs" in label and Is is not None and not np_all(Is == 0):  # Stator
            set_FEMM_circuit_prop(
                femm,
                circuits,
                label,
                Is[:, ii],
            )
        if "Circr" in label and Ir is not None and not np_all(Ir == 0):  # Rotor
            set_FEMM_circuit_prop(
                femm,
                circuits,
                label,
                Ir[:, ii],
            )



# -*- coding: utf-8 -*-
from numpy import pi, exp

from ....Classes.Arc1 import Arc1
from ....Classes.Circle import Circle
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Classes.SurfRing import SurfRing


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the Lamination

    Parameters
    ----------
    self : Lamination
        Lamination Object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    surf_list : list
        list of surfaces needed to draw the lamination

    """
    # Lamination label
    if self.is_stator:
        label = "Lamination_Stator"
    else:
        label = "Lamination_Rotor"

    label_bore = label + "_Bore_Radius"
    label_yoke = label + "_Yoke_Radius"

    if self.is_internal:
        label_int = label_yoke + "_Int"  # internal label is yoke
        label_ext = label_bore + "_Ext"  # external label is bore
    else:
        label_int = label_bore + "_Int"  # internal label is bore
        label_ext = label_yoke + "_Ext"  # external label is yoke

    surf_list = list()
    if sym == 1:  # Complete lamination
        out_surf = Circle(
            radius=self.Rext,
            label=label_ext,
            center=0,
            point_ref=self.Rint + (self.Rext - self.Rint) / 2,
        )
        in_surf = Circle(radius=self.Rint, label=label_int, center=0, point_ref=0)
        if self.Rint > 0:
            surf_list.append(
                SurfRing(
                    out_surf=out_surf,
                    in_surf=in_surf,
                    label=label,
                    point_ref=self.Rint + (self.Rext - self.Rint) / 2,
                )
            )
        else:
            surf_list.append(out_surf)
    else:  # Part of the lamination by symmetry
        Z0 = self.Rint
        Z1 = self.Rext
        Z3 = Z0 * exp(1j * 2 * pi / sym)
        Z2 = Z1 * exp(1j * 2 * pi / sym)
        curve_list = list()
        curve_list.append(Segment(Z0, Z1, label=label + "_Yoke_Side"))
        curve_list.append(
            Arc1(
                begin=Z1,
                end=Z2,
                radius=self.Rext,
                is_trigo_direction=True,
                label=label_ext,
            )
        )
        curve_list.append(Segment(Z2, Z3, label=label + "_Yoke_Side"))
        if self.Rint > 0:
            curve_list.append(
                Arc1(
                    begin=Z3,
                    end=Z0,
                    radius=-self.Rint,
                    is_trigo_direction=False,
                    label=label_int,
                )
            )
        surf_yoke = SurfLine(
            line_list=curve_list,
            label=label + "_Ext",
            point_ref=(self.Rint + (self.Rext - self.Rint) / 2) * exp(1j * pi / sym),
        )
        surf_list.append(surf_yoke)

    # Add the ventilation ducts if there is any
    for vent in self.axial_vent:
        surf_list.extend(vent.build_geometry(sym=sym, is_stator=self.is_stator))

    # apply the transformation
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW11
        A SlotW11 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    S3 = 0.5 * (self.W1 + self.W2) * (self.H2 - self.R1)
    S4 = pi * self.R1 ** 2 / 2.0 + self.R1 * (self.W2 - 2 * self.R1)

    return S3 + S4



# -*- coding: utf-8 -*-


from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from .......Classes.VentilationCirc import VentilationCirc
from .......Classes.VentilationPolar import VentilationPolar
from .......Classes.VentilationTrap import VentilationTrap
from .......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentCirc.PVentCirc import (
    PVentCirc,
)
from .......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentPolar.PVentPolar import (
    PVentPolar,
)
from .......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentTrap.PVentTrap import (
    PVentTrap,
)
from .......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.WVent.Ui_WVent import Ui_WVent

# List to convert index of combobox to slot type
INIT_INDEX = [VentilationCirc, VentilationTrap, VentilationPolar]
PAGE_INDEX = [PVentCirc, PVentTrap, PVentPolar]


class WVent(Ui_WVent, QWidget):
    """Widget to setup a Ventilation in the list"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()

    def __init__(self, lam, index):
        """Initialize the widget according the current lamination

        Parameters
        ----------
        self : WVent
            A WVent widget
        lam : Lamination
            current lamination to edit
        index : int
            Index of the ventilation in the list to update
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)
        self.obj = lam
        self.index = index

        # Avoid erase all the parameters when navigating though the vents
        self.previous_vent = dict()
        for vent_type in INIT_INDEX:
            self.previous_vent[vent_type] = None

        self.c_vent_type.setCurrentIndex(INIT_INDEX.index(type(lam.axial_vent[index])))
        # Regenerate the pages with the new values
        self.w_vent.setParent(None)
        self.w_vent = PAGE_INDEX[self.c_vent_type.currentIndex()](
            lam=lam, vent=lam.axial_vent[index]
        )
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_vent)
        self.main_layout.insertWidget(1, self.w_vent)

        # Connect the slot/signel
        self.c_vent_type.currentIndexChanged.connect(self.set_vent_type)

    def set_vent_type(self, c_index):
        """Initialize self.obj with the vent corresponding to index

        Parameters
        ----------
        self : WVent
            A WVent object
        c_index : index
            Index of the selected vent type in the combobox
        """

        # Save the vent
        vent = self.obj.axial_vent[self.index]
        self.previous_vent[type(vent)] = vent

        # Call the corresponding constructor
        if self.previous_vent[INIT_INDEX[c_index]] is None:
            # No previous vent of this type
            self.obj.axial_vent[self.index] = INIT_INDEX[c_index]()
            self.obj.axial_vent[self.index]._set_None()  # No default value
        else:  # Load the previous vent of this type
            self.obj.axial_vent[self.index] = self.previous_vent[INIT_INDEX[c_index]]

        # Update the GUI
        self.w_vent.setParent(None)
        self.w_vent = PAGE_INDEX[c_index](
            lam=self.obj, vent=self.obj.axial_vent[self.index]
        )
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_vent)
        self.main_layout.insertWidget(1, self.w_vent)

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : WVent
            A WVent object

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        return self.w_vent.check()



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError


class Slot53InterError(SlotCheckError):
    """ """

    pass


class S53_NoneError(SlotCheckError):
    """Raised when a propery of HoleM53 is None"""

    pass


class S53_Rext0CheckError(SlotCheckError):
    """ """

    pass


class S53_Rext1CheckError(SlotCheckError):
    """ """

    pass


class S53_W4CheckError(SlotCheckError):
    """ """

    pass


class S53_W5CheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-
"""File generated according to PHoleM57/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM57.Ui_PHoleM57 import Ui_PHoleM57


class Gen_PHoleM57(Ui_PHoleM57):
    def setupUi(self, PHoleM57):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PHoleM57.setupUi(self, PHoleM57)
        # Setup of in_W0
        txt = self.tr(u"""V angle""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        self.lf_W0.validator().setTop(3.15)
        txt = self.tr(u"""V angle""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_W1
        txt = self.tr(u"""Tooth width (at V bottom)""")
        self.in_W1.setWhatsThis(txt)
        self.in_W1.setToolTip(txt)

        # Setup of lf_W1
        self.lf_W1.validator().setBottom(0)
        txt = self.tr(u"""Tooth width (at V bottom)""")
        self.lf_W1.setWhatsThis(txt)
        self.lf_W1.setToolTip(txt)

        # Setup of in_W2
        txt = self.tr(u"""Distance Magnet to top of the V""")
        self.in_W2.setWhatsThis(txt)
        self.in_W2.setToolTip(txt)

        # Setup of lf_W2
        self.lf_W2.validator().setBottom(0)
        txt = self.tr(u"""Distance Magnet to top of the V""")
        self.lf_W2.setWhatsThis(txt)
        self.lf_W2.setToolTip(txt)

        # Setup of in_W3
        txt = self.tr(u"""Tooth width (at V top)""")
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""Tooth width (at V top)""")
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of in_W4
        txt = self.tr(u"""Magnet Width""")
        self.in_W4.setWhatsThis(txt)
        self.in_W4.setToolTip(txt)

        # Setup of lf_W4
        self.lf_W4.validator().setBottom(0)
        txt = self.tr(u"""Magnet Width""")
        self.lf_W4.setWhatsThis(txt)
        self.lf_W4.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Distance from the lamination Bore""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Distance from the lamination Bore""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_H2
        txt = self.tr(u"""Magnet height""")
        self.in_H2.setWhatsThis(txt)
        self.in_H2.setToolTip(txt)

        # Setup of lf_H2
        self.lf_H2.validator().setBottom(0)
        txt = self.tr(u"""Magnet height""")
        self.lf_H2.setWhatsThis(txt)
        self.lf_H2.setToolTip(txt)



# -*- coding: utf-8 -*-

import numpy as np


def get_normal(self, vertice):
    """Return the array of the normals coordinates.

    Parameters
    ----------
    self : MeshVTK
        a MeshVTK object
    indices : list
        list of the points to extract (optional)
    loc : str
        localization of the normals ("center" or "point")

    Returns
    -------
    normals: ndarray
        Normals coordinates
    """

    # u1 = vertice[1, :] - vertice[0, :]
    # u2 = vertice[2, :] - vertice[0, :]
    # n = np.cross(u1, u2)
    # n = n / np.linalg.norm(n)

    return None



# -*- coding: utf-8 -*-

from ....Functions.Geometry.inter_line_circle import inter_line_circle
from numpy import exp, pi, angle, abs as np_abs


def intersect_line(self, Z1, Z2):
    """Return a list (0, 1 or 2 complex) of coordinates of the
    intersection of the arc with a line defined by two complex

    Parameters
    ----------
    self : Arc
        An Arc object

    Returns
    -------
    Z_list: list
        Complex coordinates of the intersection (if any)
    """

    Zc = self.get_center()
    R = self.comp_radius()

    # Get intersetion between line and the full circle
    Zlist = inter_line_circle(Z1=Z1, Z2=Z2, R=R, Zc=Zc)

    # Keep only the points actually on the arc
    Zlist = [Z for Z in Zlist if self.is_on_line(Z)]

    # Order the intersection points (begin=>intersect1=>intersection2=>end)
    if len(Zlist) == 2:
        begin = self.get_begin()
        if np_abs(Zlist[0] - begin) < 1e-6:
            # First point is begin
            return Zlist
        if np_abs(Zlist[1] - begin) < 1e-6:
            # Second point is begin
            return Zlist[::-1]

        # Go to the coordinate system Zc as center, begin on X > 0 axis
        Z1 = (Zlist[0] - Zc) * exp(-1j * angle(begin - Zc))
        Z2 = (Zlist[1] - Zc) * exp(-1j * angle(begin - Zc))

        alpha = self.get_angle()
        A1 = angle(Z1) % (2 * pi)
        A2 = angle(Z2) % (2 * pi)
        # Check if Zlist needs to be reversed
        if alpha > 0 and A1 > A2:
            return Zlist[::-1]
        if alpha < 0 and A2 > A1:
            return Zlist[::-1]
    return Zlist



# -*- coding: utf-8 -*-
"""File generated according to PWSlot28/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.PWSlot28.Ui_PWSlot28 import Ui_PWSlot28


class Gen_PWSlot28(Ui_PWSlot28):
    def setupUi(self, PWSlot28):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot28.setupUi(self, PWSlot28)
        # Setup of in_W0
        txt = self.tr(u"""Slot isthmus width.""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus width.""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of unit_W0
        txt = self.tr(u"""Slot isthmus width.""")
        self.unit_W0.setWhatsThis(txt)
        self.unit_W0.setToolTip(txt)

        # Setup of in_W3
        txt = self.tr(u"""Tooth width""")
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""Tooth width""")
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of unit_W3
        txt = self.tr(u"""Tooth width""")
        self.unit_W3.setWhatsThis(txt)
        self.unit_W3.setToolTip(txt)

        # Setup of in_R1
        txt = self.tr(u"""Slot edge radius""")
        self.in_R1.setWhatsThis(txt)
        self.in_R1.setToolTip(txt)

        # Setup of lf_R1
        self.lf_R1.validator().setBottom(0)
        txt = self.tr(u"""Slot edge radius""")
        self.lf_R1.setWhatsThis(txt)
        self.lf_R1.setToolTip(txt)

        # Setup of unit_R1
        txt = self.tr(u"""Slot edge radius""")
        self.unit_R1.setWhatsThis(txt)
        self.unit_R1.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus height.""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of unit_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.unit_H0.setWhatsThis(txt)
        self.unit_H0.setToolTip(txt)

        # Setup of in_H3
        txt = self.tr(u"""Tooth height""")
        self.in_H3.setWhatsThis(txt)
        self.in_H3.setToolTip(txt)

        # Setup of lf_H3
        self.lf_H3.validator().setBottom(0)
        txt = self.tr(u"""Tooth height""")
        self.lf_H3.setWhatsThis(txt)
        self.lf_H3.setToolTip(txt)

        # Setup of unit_H3
        txt = self.tr(u"""Tooth height""")
        self.unit_H3.setWhatsThis(txt)
        self.unit_H3.setToolTip(txt)



from ....Classes._frozen import FrozenClass
from numpy import ndarray


def as_dict(self, **kwargs):
    """
    Convert this object in a json serializable dict (can be use in __init__).
    Optional keyword input parameter is for internal use only
    and may prevent json serializability.
    """
    # Get the properties inherited from ParamExplorer
    ParamExplorerSet_dict = super(type(self), self).as_dict(**kwargs)
    if self.value is None:
        ParamExplorerSet_dict["value"] = None
    else:
        ParamExplorerSet_dict["value"] = list()
        for obj in self.value:
            if isinstance(obj, FrozenClass):
                if "SciDataTool" in obj.__class__.__module__:
                    ParamExplorerSet_dict["value"].append(obj.as_dict())
                else:
                    ParamExplorerSet_dict["value"].append(obj.as_dict(**kwargs))
            elif isinstance(obj, ndarray):
                ParamExplorerSet_dict["value"].append(obj.tolist())
            else:
                ParamExplorerSet_dict["value"].append(obj)
    # The class name is added to the dict for deserialisation purpose
    # Overwrite the mother class name
    ParamExplorerSet_dict["__class__"] = "ParamExplorerSet"
    return ParamExplorerSet_dict



from ....Methods.Geometry.Circle import *


def check(self):
    """assert the Circle is correct (the radius > 0)

    Parameters
    ----------
    self : Circle
        A Circle object

    Returns
    -------
    None

    Raises
    ------
    RadiusCircleError
        The radius of a circle must be >=  0

    """
    if self.radius < 0:
        raise RadiusCircleError("The radius of a circle must be >= 0")



# -*- coding: utf-8 -*-


def __getitem__(self, idx):
    """Method to behave like a dict and get the value (and comment as a tuple)"""
    value = self._statements.__getitem__(idx)
    comment = self._comments.__getitem__(idx)
    return (value, comment)



# -*- coding: utf-8 -*-


def comp_surface_active(self, Ndisc=200):
    """Compute the Slot active surface (by numerical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : Slot
        A Slot object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    surf = self.get_surface_active()

    return surf.comp_surface(Ndisc=Ndisc)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Post/PostPlot.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Post/PostPlot
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .PostMethod import PostMethod

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Post.PostPlot.run import run
except ImportError as error:
    run = error


from ._check import InitUnKnowClassError


class PostPlot(PostMethod):
    """Post-processing to do a plot which is a method of Output or other class if attribute is not None"""

    VERSION = 1

    # cf Methods.Post.PostPlot.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use PostPlot method run: " + str(run))
            )
        )
    else:
        run = run
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        method=None,
        name="[]",
        param_list=-1,
        param_dict=-1,
        save_format="png",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "method" in list(init_dict.keys()):
                method = init_dict["method"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "param_list" in list(init_dict.keys()):
                param_list = init_dict["param_list"]
            if "param_dict" in list(init_dict.keys()):
                param_dict = init_dict["param_dict"]
            if "save_format" in list(init_dict.keys()):
                save_format = init_dict["save_format"]
        # Set the properties (value check and convertion are done in setter)
        self.method = method
        self.name = name
        self.param_list = param_list
        self.param_dict = param_dict
        self.save_format = save_format
        # Call PostMethod init
        super(PostPlot, self).__init__()
        # The class is frozen (in PostMethod init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        PostPlot_str = ""
        # Get the properties inherited from PostMethod
        PostPlot_str += super(PostPlot, self).__str__()
        PostPlot_str += 'method = "' + str(self.method) + '"' + linesep
        PostPlot_str += 'name = "' + str(self.name) + '"' + linesep
        PostPlot_str += (
            "param_list = "
            + linesep
            + str(self.param_list).replace(linesep, linesep + "\t")
            + linesep
        )
        PostPlot_str += "param_dict = " + str(self.param_dict) + linesep
        PostPlot_str += 'save_format = "' + str(self.save_format) + '"' + linesep
        return PostPlot_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from PostMethod
        if not super(PostPlot, self).__eq__(other):
            return False
        if other.method != self.method:
            return False
        if other.name != self.name:
            return False
        if other.param_list != self.param_list:
            return False
        if other.param_dict != self.param_dict:
            return False
        if other.save_format != self.save_format:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from PostMethod
        diff_list.extend(super(PostPlot, self).compare(other, name=name))
        if other._method != self._method:
            diff_list.append(name + ".method")
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._param_list != self._param_list:
            diff_list.append(name + ".param_list")
        if other._param_dict != self._param_dict:
            diff_list.append(name + ".param_dict")
        if other._save_format != self._save_format:
            diff_list.append(name + ".save_format")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from PostMethod
        S += super(PostPlot, self).__sizeof__()
        S += getsizeof(self.method)
        S += getsizeof(self.name)
        if self.param_list is not None:
            for value in self.param_list:
                S += getsizeof(value)
        if self.param_dict is not None:
            for key, value in self.param_dict.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.save_format)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from PostMethod
        PostPlot_dict = super(PostPlot, self).as_dict(**kwargs)
        PostPlot_dict["method"] = self.method
        PostPlot_dict["name"] = self.name
        PostPlot_dict["param_list"] = (
            self.param_list.copy() if self.param_list is not None else None
        )
        PostPlot_dict["param_dict"] = (
            self.param_dict.copy() if self.param_dict is not None else None
        )
        PostPlot_dict["save_format"] = self.save_format
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        PostPlot_dict["__class__"] = "PostPlot"
        return PostPlot_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.method = None
        self.name = None
        self.param_list = None
        self.param_dict = None
        self.save_format = None
        # Set to None the properties inherited from PostMethod
        super(PostPlot, self)._set_None()

    def _get_method(self):
        """getter of method"""
        return self._method

    def _set_method(self, value):
        """setter of method"""
        check_var("method", value, "str")
        self._method = value

    method = property(
        fget=_get_method,
        fset=_set_method,
        doc=u"""Full path of the plot method to call except Output (ex: "plot_2D_Data", "mag.meshsolution.plot_contour")

        :Type: str
        """,
    )

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the plot to use when saving the figure after plotting

        :Type: str
        """,
    )

    def _get_param_list(self):
        """getter of param_list"""
        return self._param_list

    def _set_param_list(self, value):
        """setter of param_list"""
        if type(value) is int and value == -1:
            value = list()
        check_var("param_list", value, "list")
        self._param_list = value

    param_list = property(
        fget=_get_param_list,
        fset=_set_param_list,
        doc=u"""Dictionnary of parameters to pass to the plot method when executing it

        :Type: list
        """,
    )

    def _get_param_dict(self):
        """getter of param_dict"""
        return self._param_dict

    def _set_param_dict(self, value):
        """setter of param_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("param_dict", value, "dict")
        self._param_dict = value

    param_dict = property(
        fget=_get_param_dict,
        fset=_set_param_dict,
        doc=u"""Dictionnary of parameters to pass to the plot method when executing it

        :Type: dict
        """,
    )

    def _get_save_format(self):
        """getter of save_format"""
        return self._save_format

    def _set_save_format(self, value):
        """setter of save_format"""
        check_var("save_format", value, "str")
        self._save_format = value

    save_format = property(
        fget=_get_save_format,
        fset=_set_save_format,
        doc=u"""File format extension ("png", "svg", "eps") in which to save the figure. The PostPlot automatically saves the figure in the results folder. The user can specify a different folder by specifying "save_path"=path_str or not save the figure by specifying "save_path"=None in param_dict, if the plot_method enables it.

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-


def comp_number_phase_eq(self):
    """Compute the equivalent number of phase

    Parameters
    ----------
    self : LamSquirrelCage
        A LamSquirrelCage object

    Returns
    -------
    qb: float
        Zs/p
    """

    return self.slot.Zs / float(self.winding.p)



# -*- coding: utf-8 -*-

import numpy as np


def shape_function(self, points, nb_pt):
    """ Return the values of linear shape functions in reference triangle for a given point """
    # nb_pt = points.shape[0]
    values = np.zeros([nb_pt, 1, 3], dtype=float)
    for i in range(nb_pt):

        if nb_pt == 1:
            [x, y] = points[0:2]
        else:
            [x, y] = points[i, 0:2]

        # if (x >= 0) and (y >= 0) and (1 - x - y >= 0):
        values[i, 0, 0] = 1 - x - y
        values[i, 0, 1] = x
        values[i, 0, 2] = y

    size = 3

    return values, size



from ..Load.import_class import import_class

unit_dict = {
    "time": "s",
    "angle": "rad",
    "freqs": "Hz",
    "wavenumber": "",
    "phase": "",
    "z": "m",
    "radius": "m",
    "distance": "m",
}

norm_dict = {
    "elec_order": "Electrical order []",
    "mech_order": "Mechanical order []",
    "space_order": "Space order []",
    "distance": "Distance [m]",
    "angle_rotor": "Rotor mechanical angle [°]",
}

axes_dict = {
    "freqs": "frequency",
}

fft_dict = {
    "time": "freqs",
    "angle": "wavenumber",
}

ifft_dict = {
    "freqs": "time",
    "wavenumber": "angle",
}

## Paramater for Schematics plot
P_FONT_SIZE = 12  # Point Font size
SC_FONT_SIZE = 12  # Schematics Font size
TEXT_BOX = dict(  # Parameter of the text box
    boxstyle="round",
    ec=(0.0, 0.0, 0.0),
    fc=(1.0, 1.0, 1.0),
)
# Arrow parameters
ARROW_WIDTH = 2
ARROW_COLOR = "black"
# Schematics lines
SC_LINE_COLOR = "black"
SC_LINE_STYLE = "dotted"
SC_LINE_WIDTH = 1
# Main lines
MAIN_LINE_COLOR = "0.5"  # Gray
MAIN_LINE_STYLE = "dotted"
MAIN_LINE_WIDTH = 1


def plot_quote(Z1, Zlim1, Zlim2, Z2, offset_label=0, fig=None, ax=None, label=None):
    """Function to plot a "quote" for the scematics"""
    Segment = import_class("pyleecan.Classes", "Segment")
    line1 = Segment(Z1, Zlim1)
    line2 = Segment(Zlim1, Zlim2)
    line3 = Segment(Zlim2, Z2)
    line1.plot(
        fig=fig,
        ax=ax,
        color=SC_LINE_COLOR,
        linestyle=SC_LINE_STYLE,
        linewidth=SC_LINE_WIDTH,
    )
    line2.plot(
        fig=fig,
        ax=ax,
        color=ARROW_COLOR,
        linewidth=ARROW_WIDTH,
        label=label,
        offset_label=offset_label,
        is_arrow=True,
        fontsize=SC_FONT_SIZE,
    )
    line3.plot(
        fig=fig,
        ax=ax,
        color=SC_LINE_COLOR,
        linestyle=SC_LINE_STYLE,
        linewidth=SC_LINE_WIDTH,
    )



# -*- coding: utf-8 -*-

from ....Classes.OutElec import OutElec
from ....Methods.Simulation.Input import InputError
from numpy import ndarray


def gen_input(self):
    """Generate the input for the electrical module (time/space discretization)

    Parameters
    ----------
    self : Input
        An Input object
    """
    # Get the simulation and check output
    if self.parent is None:
        raise InputError("ERROR: Input object should be inside a Simulation object")
    simu = self.parent

    if self.parent.parent is None:
        raise InputError(
            "ERROR: Input parent error:"
            + " The Simulation object must be in an Output object to run"
        )

    # Create the correct Output object
    output = OutElec()
    self.comp_axes(machine=simu.machine, N0=self.N0)

    # Save the Output in the correct place
    self.parent.parent.elec = output



# -*- coding: utf-8 -*-
"""File generated by generate_code() - 
WARNING! All changes made in this file will be lost!
"""

from ..Classes.import_all import *

load_switch = {
    "Arc": Arc,
    "Arc1": Arc1,
    "Arc2": Arc2,
    "Arc3": Arc3,
    "Bore": Bore,
    "BoreFlower": BoreFlower,
    "BoreUD": BoreUD,
    "CellMat": CellMat,
    "Circle": Circle,
    "CondType11": CondType11,
    "CondType12": CondType12,
    "CondType21": CondType21,
    "CondType22": CondType22,
    "Conductor": Conductor,
    "DXFImport": DXFImport,
    "DataKeeper": DataKeeper,
    "Drive": Drive,
    "DriveWave": DriveWave,
    "EEC": EEC,
    "EEC_PMSM": EEC_PMSM,
    "EEC_SCIM": EEC_SCIM,
    "Electrical": Electrical,
    "Elmer": Elmer,
    "ElmerResults": ElmerResults,
    "ElmerResultsVTU": ElmerResultsVTU,
    "FPGNSeg": FPGNSeg,
    "FPGNTri": FPGNTri,
    "FluxLink": FluxLink,
    "FluxLinkFEMM": FluxLinkFEMM,
    "Force": Force,
    "ForceMT": ForceMT,
    "Frame": Frame,
    "GUIOption": GUIOption,
    "GaussPoint": GaussPoint,
    "Hole": Hole,
    "HoleM50": HoleM50,
    "HoleM51": HoleM51,
    "HoleM52": HoleM52,
    "HoleM53": HoleM53,
    "HoleM54": HoleM54,
    "HoleM57": HoleM57,
    "HoleM58": HoleM58,
    "HoleMag": HoleMag,
    "HoleUD": HoleUD,
    "Import": Import,
    "ImportData": ImportData,
    "ImportGenMatrixSin": ImportGenMatrixSin,
    "ImportGenToothSaw": ImportGenToothSaw,
    "ImportGenVectLin": ImportGenVectLin,
    "ImportGenVectSin": ImportGenVectSin,
    "ImportMatlab": ImportMatlab,
    "ImportMatrix": ImportMatrix,
    "ImportMatrixVal": ImportMatrixVal,
    "ImportMatrixXls": ImportMatrixXls,
    "ImportVectorField": ImportVectorField,
    "IndMag": IndMag,
    "IndMagFEMM": IndMagFEMM,
    "Input": Input,
    "InputCurrent": InputCurrent,
    "InputElec": InputElec,
    "InputFlux": InputFlux,
    "InputForce": InputForce,
    "Interpolation": Interpolation,
    "LamHole": LamHole,
    "LamSlot": LamSlot,
    "LamSlotMag": LamSlotMag,
    "LamSlotMulti": LamSlotMulti,
    "LamSlotWind": LamSlotWind,
    "LamSquirrelCage": LamSquirrelCage,
    "Lamination": Lamination,
    "Line": Line,
    "Loss": Loss,
    "LossModel": LossModel,
    "LossModelBertotti": LossModelBertotti,
    "LossModelWinding": LossModelWinding,
    "Machine": Machine,
    "MachineAsync": MachineAsync,
    "MachineDFIM": MachineDFIM,
    "MachineIPMSM": MachineIPMSM,
    "MachineSCIM": MachineSCIM,
    "MachineSIPMSM": MachineSIPMSM,
    "MachineSRM": MachineSRM,
    "MachineSyRM": MachineSyRM,
    "MachineSync": MachineSync,
    "MachineUD": MachineUD,
    "MachineWRSM": MachineWRSM,
    "MagElmer": MagElmer,
    "MagFEMM": MagFEMM,
    "Magnet": Magnet,
    "Magnetics": Magnetics,
    "MatEconomical": MatEconomical,
    "MatElectrical": MatElectrical,
    "MatHT": MatHT,
    "MatMagnetics": MatMagnetics,
    "MatStructural": MatStructural,
    "Material": Material,
    "Mesh": Mesh,
    "MeshMat": MeshMat,
    "MeshSolution": MeshSolution,
    "MeshVTK": MeshVTK,
    "Mode": Mode,
    "Notch": Notch,
    "NotchEvenDist": NotchEvenDist,
    "OptiConstraint": OptiConstraint,
    "OptiDesignVar": OptiDesignVar,
    "OptiGenAlg": OptiGenAlg,
    "OptiGenAlgNsga2Deap": OptiGenAlgNsga2Deap,
    "OptiObjective": OptiObjective,
    "OptiProblem": OptiProblem,
    "OptiSolver": OptiSolver,
    "OutElec": OutElec,
    "OutForce": OutForce,
    "OutGeo": OutGeo,
    "OutGeoLam": OutGeoLam,
    "OutInternal": OutInternal,
    "OutLoss": OutLoss,
    "OutMag": OutMag,
    "OutMagElmer": OutMagElmer,
    "OutMagFEMM": OutMagFEMM,
    "OutPost": OutPost,
    "OutStruct": OutStruct,
    "Output": Output,
    "ParamExplorer": ParamExplorer,
    "ParamExplorerSet": ParamExplorerSet,
    "PointMat": PointMat,
    "PolarArc": PolarArc,
    "Post": Post,
    "PostFunction": PostFunction,
    "PostMethod": PostMethod,
    "PostPlot": PostPlot,
    "RefCell": RefCell,
    "RefLine3": RefLine3,
    "RefQuad4": RefQuad4,
    "RefQuad9": RefQuad9,
    "RefSegmentP1": RefSegmentP1,
    "RefTriangle3": RefTriangle3,
    "RefTriangle6": RefTriangle6,
    "ScalarProduct": ScalarProduct,
    "ScalarProductL2": ScalarProductL2,
    "Section": Section,
    "Segment": Segment,
    "Shaft": Shaft,
    "Simu1": Simu1,
    "Simulation": Simulation,
    "Slot": Slot,
    "Slot19": Slot19,
    "SlotCirc": SlotCirc,
    "SlotDC": SlotDC,
    "SlotM10": SlotM10,
    "SlotM11": SlotM11,
    "SlotM12": SlotM12,
    "SlotM13": SlotM13,
    "SlotM14": SlotM14,
    "SlotM15": SlotM15,
    "SlotM16": SlotM16,
    "SlotM17": SlotM17,
    "SlotUD": SlotUD,
    "SlotUD2": SlotUD2,
    "SlotW10": SlotW10,
    "SlotW11": SlotW11,
    "SlotW12": SlotW12,
    "SlotW13": SlotW13,
    "SlotW14": SlotW14,
    "SlotW15": SlotW15,
    "SlotW16": SlotW16,
    "SlotW21": SlotW21,
    "SlotW22": SlotW22,
    "SlotW23": SlotW23,
    "SlotW24": SlotW24,
    "SlotW25": SlotW25,
    "SlotW26": SlotW26,
    "SlotW27": SlotW27,
    "SlotW28": SlotW28,
    "SlotW29": SlotW29,
    "SlotW60": SlotW60,
    "SlotW61": SlotW61,
    "Solution": Solution,
    "SolutionData": SolutionData,
    "SolutionMat": SolutionMat,
    "SolutionVector": SolutionVector,
    "SolverInputFile": SolverInputFile,
    "StructElmer": StructElmer,
    "Structural": Structural,
    "SurfLine": SurfLine,
    "SurfRing": SurfRing,
    "Surface": Surface,
    "Trapeze": Trapeze,
    "Unit": Unit,
    "VarLoad": VarLoad,
    "VarLoadCurrent": VarLoadCurrent,
    "VarParam": VarParam,
    "VarSimu": VarSimu,
    "VentilationCirc": VentilationCirc,
    "VentilationPolar": VentilationPolar,
    "VentilationTrap": VentilationTrap,
    "Winding": Winding,
    "WindingCW1L": WindingCW1L,
    "WindingCW2LR": WindingCW2LR,
    "WindingCW2LT": WindingCW2LT,
    "WindingDW1L": WindingDW1L,
    "WindingDW2L": WindingDW2L,
    "WindingSC": WindingSC,
    "WindingUD": WindingUD,
    "XOutput": XOutput,
}



from ....Methods.Slot.Slot import SlotCheckError


class S51_NoneError(SlotCheckError):
    """Raised when a propery of HoleM51 is None"""

    pass


class S51_WCheckError(SlotCheckError):
    """ """

    pass


class S51_RHCheckError(SlotCheckError):
    """ """

    pass


class S51_AlphaCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotM16 import *


def check(self):
    """Check that the SlotM16 object is correct

    Parameters
    ----------
    self : SlotM16
        A SlotM16 object

    Returns
    -------
    None

    Raises
    -------
    S29_W01CheckError
        You must have W0 < W1

    """
    if self.W1 <= self.W0:
        raise S16_W01CheckError("You must have W0 < W1")



# -*- coding: utf-8 -*-

import quadpy


def get_gauss_points(self):
    """ Return the gauss points and weights for Triangle3 cell """

    nb_gauss_points = self.nb_gauss_point
    scheme = quadpy.tn.grundmann_moeller(2, nb_gauss_points)
    gauss_pts = scheme.points
    weights = scheme.weights

    return gauss_pts, weights, nb_gauss_points



from os.path import basename, splitext

from numpy import zeros, pi, roll, cos, sin

# from scipy.interpolate import interp1d

from ....Classes._FEMMHandler import _FEMMHandler
from ....Functions.FEMM.update_FEMM_simulation import update_FEMM_simulation
from ....Functions.FEMM.comp_FEMM_torque import comp_FEMM_torque
from ....Functions.FEMM.comp_FEMM_Phi_wind import comp_FEMM_Phi_wind


def solve_FEMM(
    self,
    femm,
    output,
    out_dict,
    FEMM_dict,
    sym,
    Nt,
    angle,
    Is,
    Ir,
    angle_rotor,
    is_close_femm,
    filename=None,
    start_t=0,
    end_t=None,
):
    """
    Solve FEMM model to calculate airgap flux density, torque instantaneous/average/ripple values,
    flux induced in stator windings and flux density, field and permeability maps

    Parameters
    ----------
    self: MagFEMM
        A MagFEMM object
    femm: _FEMMHandler
        Object to handle FEMM
    output: Output
        An Output object
    out_dict: dict
        Dict containing the following quantities to update for each time step:
            Br : ndarray
                Airgap radial flux density (Nt,Na) [T]
            Bt : ndarray
                Airgap tangential flux density (Nt,Na) [T]
            Tem : ndarray
                Electromagnetic torque over time (Nt,) [Nm]
            Phi_wind : list of ndarray # TODO should it rather be a dict with lam label?
                List of winding flux with respect to Machine.get_lamlist (qs,Nt) [Wb]
    FEMM_dict : dict
        Dict containing FEMM model parameters
    sym: int
        Spatial symmetry factor
    Nt: int
        Number of time steps for calculation
    angle: ndarray
        Angle vector for calculation
    Is : ndarray
        Stator current matrix (qs,Nt) [A]
    Ir : ndarray
        Stator current matrix (qs,Nt) [A]
    angle_rotor: ndarray
        Rotor angular position vector (Nt,)
    is_close_femm: bool
        True to close FEMM handler in the end
    filename: str
        Path to FEMM model to open
    start_t: int
        Index of first time step (0 by default, used for parallelization)
    end_t: int
        Index of last time step (Nt by default, used for parallelization)

    Returns
    -------

    """
    # Open FEMM file if not None, else it is already open
    if filename is not None:
        try:
            # Open the document
            femm.openfemm(1)
        except:
            # Create a new FEMM handler in case of parallelization on another FEMM instance
            femm = _FEMMHandler()
            output.mag.internal.handler_list.append(femm)
            # Open the document
            femm.openfemm(1)

        # Import FEMM file
        femm.opendocument(filename)

    # Take last time step at Nt by default
    if end_t is None:
        end_t = Nt

    # Number of angular steps
    Na = angle.size

    # Loading parameters for readibility
    machine = output.simu.machine
    Rag = self.Rag_enforced
    if Rag is None:
        Rag = machine.comp_Rgap_mec()

    L1 = machine.stator.comp_length()
    save_path = self.get_path_save(output)
    is_internal_rotor = machine.rotor.is_internal
    if "Phi_wind" in out_dict:
        qs = {}
        Npcpp = {}
        for key in out_dict["Phi_wind"].keys():
            lam = machine.get_lam_by_label(key)
            qs[key] = lam.winding.qs  # Winding phase number
            Npcpp[key] = lam.winding.Npcpp  # parallel paths

    # Account for initial angular shift of stator and rotor and apply it to the sliding band
    angle_shift = self.angle_rotor_shift - self.angle_stator_shift

    B_elem = None
    H_elem = None
    mu_elem = None
    meshFEMM = None
    groups = None

    # Compute the data for each time step
    for ii in range(start_t, end_t):
        if Nt > 1:
            self.get_logger().info(
                "Solving time step " + str(ii + 1) + " / " + str(Nt) + " in FEMM"
            )
        else:
            self.get_logger().info("Computing Airgap Flux in FEMM")
        # Update rotor position and currents
        update_FEMM_simulation(
            femm=femm,
            circuits=FEMM_dict["circuits"],
            is_sliding_band=self.is_sliding_band,
            is_internal_rotor=is_internal_rotor,
            angle_rotor=angle_rotor + angle_shift,
            Is=Is,
            Ir=Ir,
            ii=ii,
        )
        # try "previous solution" for speed up of FEMM calculation
        if self.is_sliding_band:
            try:
                base = basename(self.get_path_save_fem(output))
                ans_file = splitext(base)[0] + ".ans"
                femm.mi_setprevious(ans_file, 0)
            except:
                pass

        # Run the computation
        femm.mi_analyze()

        # Load results
        femm.mi_loadsolution()

        # Get the flux result
        if self.is_sliding_band:
            for jj in range(Na):
                out_dict["Br"][ii, jj], out_dict["Bt"][ii, jj] = femm.mo_getgapb(
                    "bc_ag2", angle[jj] * 180 / pi
                )
        else:
            for jj in range(Na):
                B = femm.mo_getb(Rag * cos(angle[jj]), Rag * sin(angle[jj]))
                out_dict["Br"][ii, jj] = B[0] * cos(angle[jj]) + B[1] * sin(angle[jj])
                out_dict["Bt"][ii, jj] = -B[0] * sin(angle[jj]) + B[1] * cos(angle[jj])

        # Compute the torque
        out_dict["Tem"][ii] = comp_FEMM_torque(femm, FEMM_dict, sym=sym)

        if "Phi_wind" in out_dict:
            # Phi_wind computation
            # TODO fix inconsistency for multi lam machines here
            for key in out_dict["Phi_wind"].keys():
                out_dict["Phi_wind"][key][ii, :] = comp_FEMM_Phi_wind(
                    femm,
                    qs[key],
                    Npcpp[key],
                    is_stator=machine.get_lam_by_label(key).is_stator,
                    Lfemm=FEMM_dict["Lfemm"],
                    L1=L1,
                    sym=sym,
                )

        # Load mesh data & solution
        if (self.is_sliding_band or Nt == 1) and (self.is_get_meshsolution):
            # Get mesh data and magnetic quantities from .ans file
            tmpmeshFEMM, tmpB, tmpH, tmpmu, tmpgroups = self.get_meshsolution(
                femm,
                save_path,
                j_t0=ii,
                id_worker=start_t,
                is_get_mesh=ii == start_t,
            )

            # Store magnetic flux density, field and relative permeability for the current time step

            # Initialize mesh and magnetic quantities for first time step
            if ii == start_t:
                meshFEMM = [tmpmeshFEMM]
                groups = tmpgroups
                Nelem = meshFEMM[0].cell["triangle"].nb_cell
                Nt0 = end_t - start_t
                B_elem = zeros([Nt0, Nelem, 3])
                H_elem = zeros([Nt0, Nelem, 3])
                mu_elem = zeros([Nt0, Nelem])

            # Shift time index ii in case start_t is not 0 (parallelization)
            ii0 = ii - start_t

            B_elem[ii0, :, 0:2] = tmpB
            H_elem[ii0, :, 0:2] = tmpH
            mu_elem[ii0, :] = tmpmu

    # Shift to take into account stator position
    if self.angle_stator_shift != 0:
        roll_id = int(self.angle_stator_shift * Na / (2 * pi))
        out_dict["Br"] = roll(out_dict["Br"], roll_id, axis=1)
        out_dict["Bt"] = roll(out_dict["Bt"], roll_id, axis=1)

        # # Interpolate on updated angular position # TODO to improve accuracy
        # angle_new = (angle - self.angle_stator_shift) % (2 * pi / sym)
        # out_dict["Br"] = interp1d(append(angle, 2 * pi / sym), append(out_dict["Br"], out_dict["Br"][:,0]), axis=1)[angle_new]
        # out_dict["Bt"] = interp1d(append(angle, 2 * pi / sym), append(out_dict["Bt"], out_dict["Bt"][:,0]), axis=1)[angle_new]

    # Close FEMM handler
    if is_close_femm:
        femm.closefemm()
        output.mag.internal.handler_list.remove(femm)

    out_dict["Rag"] = Rag

    return B_elem, H_elem, mu_elem, meshFEMM, groups



# -*- coding: utf-8 -*-

# File generated according to WMatSelect.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_WMatSelect(object):
    def setupUi(self, WMatSelect):
        if not WMatSelect.objectName():
            WMatSelect.setObjectName(u"WMatSelect")
        WMatSelect.resize(283, 30)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WMatSelect.sizePolicy().hasHeightForWidth())
        WMatSelect.setSizePolicy(sizePolicy)
        WMatSelect.setMinimumSize(QSize(0, 0))
        self.horizontalLayout = QHBoxLayout(WMatSelect)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(4, 2, 4, 2)
        self.in_mat_type = QLabel(WMatSelect)
        self.in_mat_type.setObjectName(u"in_mat_type")

        self.horizontalLayout.addWidget(self.in_mat_type)

        self.c_mat_type = QComboBox(WMatSelect)
        self.c_mat_type.addItem("")
        self.c_mat_type.addItem("")
        self.c_mat_type.addItem("")
        self.c_mat_type.setObjectName(u"c_mat_type")

        self.horizontalLayout.addWidget(self.c_mat_type)

        self.b_matlib = QPushButton(WMatSelect)
        self.b_matlib.setObjectName(u"b_matlib")

        self.horizontalLayout.addWidget(self.b_matlib)

        QWidget.setTabOrder(self.c_mat_type, self.b_matlib)

        self.retranslateUi(WMatSelect)

        QMetaObject.connectSlotsByName(WMatSelect)

    # setupUi

    def retranslateUi(self, WMatSelect):
        WMatSelect.setWindowTitle(
            QCoreApplication.translate("WMatSelect", u"Form", None)
        )
        self.in_mat_type.setText(
            QCoreApplication.translate("WMatSelect", u"mat_type :", None)
        )
        self.c_mat_type.setItemText(
            0, QCoreApplication.translate("WMatSelect", u"M400-50A", None)
        )
        self.c_mat_type.setItemText(
            1, QCoreApplication.translate("WMatSelect", u"M350-50A", None)
        )
        self.c_mat_type.setItemText(
            2, QCoreApplication.translate("WMatSelect", u"M330-35A", None)
        )

        self.b_matlib.setText(
            QCoreApplication.translate("WMatSelect", u"Edit Materials", None)
        )

    # retranslateUi



# -*- coding: utf-8 -*-

from ....Methods.Machine.LamSlot import *


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self : LamSlot
        A LamSlot object

    Returns
    -------
    None

    Raises
    _______
    LSC_SlotTooLong
        The Slot is too long for the lamination (HYoke <0)

    LSC_OverlappingSlot
        The Lamination has overlapping slot

    """

    super(type(self), self).check()

    self.slot.check()

    if self.comp_height_yoke() < 0:
        raise LSC_SlotTooLong("The Slot is too long for the lamination " "(HYoke <0)")

    """
    Wt = self.slot.comp_tooth_widths()["WTooth_min"]

    if Wt < 0:
        raise LSC_OverlappingSlot("The Lamination has overlapping slot")
    """



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/MachineWRSM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/MachineWRSM
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .MachineSync import MachineSync

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.MachineWRSM.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Machine.MachineWRSM.get_machine_type import get_machine_type
except ImportError as error:
    get_machine_type = error


from ._check import InitUnKnowClassError
from .LamSlotWind import LamSlotWind
from .Frame import Frame
from .Shaft import Shaft


class MachineWRSM(MachineSync):
    """Wound Rotor Synchronous Machine"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.MachineWRSM.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use MachineWRSM method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Machine.MachineWRSM.get_machine_type
    if isinstance(get_machine_type, ImportError):
        get_machine_type = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MachineWRSM method get_machine_type: "
                    + str(get_machine_type)
                )
            )
        )
    else:
        get_machine_type = get_machine_type
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        rotor=-1,
        stator=-1,
        frame=-1,
        shaft=-1,
        name="default_machine",
        desc="",
        type_machine=1,
        logger_name="Pyleecan.Machine",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "rotor" in list(init_dict.keys()):
                rotor = init_dict["rotor"]
            if "stator" in list(init_dict.keys()):
                stator = init_dict["stator"]
            if "frame" in list(init_dict.keys()):
                frame = init_dict["frame"]
            if "shaft" in list(init_dict.keys()):
                shaft = init_dict["shaft"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "type_machine" in list(init_dict.keys()):
                type_machine = init_dict["type_machine"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.rotor = rotor
        self.stator = stator
        # Call MachineSync init
        super(MachineWRSM, self).__init__(
            frame=frame,
            shaft=shaft,
            name=name,
            desc=desc,
            type_machine=type_machine,
            logger_name=logger_name,
        )
        # The class is frozen (in MachineSync init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MachineWRSM_str = ""
        # Get the properties inherited from MachineSync
        MachineWRSM_str += super(MachineWRSM, self).__str__()
        if self.rotor is not None:
            tmp = self.rotor.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MachineWRSM_str += "rotor = " + tmp
        else:
            MachineWRSM_str += "rotor = None" + linesep + linesep
        if self.stator is not None:
            tmp = self.stator.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            MachineWRSM_str += "stator = " + tmp
        else:
            MachineWRSM_str += "stator = None" + linesep + linesep
        return MachineWRSM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from MachineSync
        if not super(MachineWRSM, self).__eq__(other):
            return False
        if other.rotor != self.rotor:
            return False
        if other.stator != self.stator:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from MachineSync
        diff_list.extend(super(MachineWRSM, self).compare(other, name=name))
        if (other.rotor is None and self.rotor is not None) or (
            other.rotor is not None and self.rotor is None
        ):
            diff_list.append(name + ".rotor None mismatch")
        elif self.rotor is not None:
            diff_list.extend(self.rotor.compare(other.rotor, name=name + ".rotor"))
        if (other.stator is None and self.stator is not None) or (
            other.stator is not None and self.stator is None
        ):
            diff_list.append(name + ".stator None mismatch")
        elif self.stator is not None:
            diff_list.extend(self.stator.compare(other.stator, name=name + ".stator"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from MachineSync
        S += super(MachineWRSM, self).__sizeof__()
        S += getsizeof(self.rotor)
        S += getsizeof(self.stator)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from MachineSync
        MachineWRSM_dict = super(MachineWRSM, self).as_dict(**kwargs)
        if self.rotor is None:
            MachineWRSM_dict["rotor"] = None
        else:
            MachineWRSM_dict["rotor"] = self.rotor.as_dict(**kwargs)
        if self.stator is None:
            MachineWRSM_dict["stator"] = None
        else:
            MachineWRSM_dict["stator"] = self.stator.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MachineWRSM_dict["__class__"] = "MachineWRSM"
        return MachineWRSM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.rotor is not None:
            self.rotor._set_None()
        if self.stator is not None:
            self.stator._set_None()
        # Set to None the properties inherited from MachineSync
        super(MachineWRSM, self)._set_None()

    def _get_rotor(self):
        """getter of rotor"""
        return self._rotor

    def _set_rotor(self, value):
        """setter of rotor"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "rotor"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = LamSlotWind()
        check_var("rotor", value, "LamSlotWind")
        self._rotor = value

        if self._rotor is not None:
            self._rotor.parent = self

    rotor = property(
        fget=_get_rotor,
        fset=_set_rotor,
        doc=u"""Machine's Rotor

        :Type: LamSlotWind
        """,
    )

    def _get_stator(self):
        """getter of stator"""
        return self._stator

    def _set_stator(self, value):
        """setter of stator"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "stator"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = LamSlotWind()
        check_var("stator", value, "LamSlotWind")
        self._stator = value

        if self._stator is not None:
            self._stator.parent = self

    stator = property(
        fget=_get_stator,
        fset=_set_stator,
        doc=u"""Machine's Stator

        :Type: LamSlotWind
        """,
    )



# -*- coding: utf-8 -*-


def check_cstr(solver, indiv):
    """Check the number of constraints violations of the individual

    Parameters
    ----------
    solver : Solver
        Global optimization problem solver
    indiv : individual
        Individual of the population

    Returns
    -------
    is_feasible : bool
        Individual feasibility
    """

    # Non valid simulation violate every constraints
    if indiv.is_simu_valid == False:
        indiv.cstr_viol = len(solver.problem.constraint)
        return True  # To not add errors to infeasible

    # Browse constraints
    for constraint in solver.problem.constraint:
        # Compute value to compare
        var_val = constraint.get_variable(indiv.output)

        # Compare the value with the constraint
        type_const = constraint.type_const

        if type_const == "<=":
            if var_val > constraint.value:
                indiv.cstr_viol += 1
        elif type_const in ["==", "="]:
            if var_val != constraint.value:
                indiv.cstr_viol += 1
        elif type_const == ">=":
            if var_val < constraint.value:
                indiv.cstr_viol += 1
        elif type_const == "<":
            if var_val >= constraint.value:
                indiv.cstr_viol += 1
        elif type_const == ">":
            if var_val <= constraint.value:
                indiv.cstr_viol += 1
        else:
            raise ValueError("Wrong type of constraint")

    return indiv.cstr_viol == 0



# -*- coding: utf-8 -*-
from deap import base, creator, tools
from ....Classes.Output import Output
from ....Classes.XOutput import XOutput
from ....Classes.VarSimu import VarSimu


def create_toolbox(self):
    """OptiGenAlgNsga2Deap method to create DEAP toolbox
    Parameters
    ----------
    self : OptiGenAlgNsga2Deap

    Returns
    -------
    self : OptiGenAlgNsga2Deap
        OptiGenAlgNsga2Deap with toolbox created
    """

    # Create toolbox
    self.toolbox = base.Toolbox()

    # Create Fitness and individual
    creator.create(
        "FitnessMin", base.Fitness, weights=[-1 for _ in self.problem.obj_func]
    )
    creator.create("Individual", list, typecode="d", fitness=creator.FitnessMin)

    self.toolbox.register("creator", creator.Individual)

    # Create default output
    if isinstance(self.problem.simu.parent, Output):
        output = self.problem.simu.parent
    elif isinstance(
        self.problem.simu.var_simu, VarSimu
    ):  # Optimization of a multi-simulation
        output = XOutput(simu=self.problem.simu.copy())
    else:
        output = Output(simu=self.problem.simu.copy())

    # Register individual and population
    self.toolbox.register(
        "individual",
        create_indiv,
        self.toolbox.creator,
        output,
        self.problem.design_var,
    )

    self.toolbox.register("population", tools.initRepeat, list, self.toolbox.individual)


def create_indiv(create, output, design_var_list):
    """Create individual using DEAP tools

    Parameters
    ----------
    creator : function
        function to create the individual
    output : ....Classes.Output
        output of the individual
    design_var_list : list
        Design variables

    Returns:
    --------
    indiv : list
        individual
    """

    # Extract design variables
    var = []

    for design_var in design_var_list:
        # Generate the first value
        value = design_var.get_value(design_var.space)
        design_var.setter(output.simu, value)
        var.append(value)

    ind = create(var)

    # Store the design_var_name_list
    ind.design_var_name_list = [dv.name for dv in design_var_list]

    # Store setters in a list
    ind.setter_list = [dv.setter for dv in design_var_list]

    # Store the design variables
    ind.design_var = design_var_list

    # Store the simulation validity
    ind.is_simu_valid = False

    # Store the number of constraints violations
    ind.cstr_viol = 0

    # Output with the design variables set
    ind.output = type(output)(simu=output.simu.as_dict())

    return ind



from numpy import arcsin, exp, pi, sqrt
from ....Classes.Arc1 import Arc1


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW28
        A SlotW28 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()

    # alpha is the angle to rotate Z0 so ||Z1,Z8|| = W0
    alpha = float(arcsin(self.W0 / (2 * Rbo)))
    slot_pitch = 2 * pi / self.Zs

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z8 = Z0 * exp(-1j * alpha)

    if self.is_outwards():
        Z7 = Z8 + self.H0
        # Rotation to get the tooth on X axis
        Z7 = Z7 * exp(1j * slot_pitch / 2)
        Z8 = Z8 * exp(1j * slot_pitch / 2)
        # Z7 = x7 + 1j*y7
        # Z6 = x + 1j * W3/2
        # C2,Z6 _|_ Z6,Z5 => Re(C2) = Re(Z6)
        # ||Z6,zc2|| = R1 => Zc2 = x + 1j*(W3/2+R1)
        # ||Z7,zc2||² = R1² => (x7-x)²+ (y7-(W3/2+R1))² = R1²

        # x² - 2*x7 x + (x7²+(y7-(W3/2+R1))²-R1²) = 0
        # D = 4*x7² - 4*(x7²+(y7-(W3/2+R1))²-R1²) = -4((y7-(W3/2+R1))²-R1²)
        # x = x7 + sqrt(-4((y7-(W3/2+R1))²-R1²))/2
        Z6 = (
            Z7.real
            + sqrt(-4 * ((Z7.imag - (self.W3 / 2.0 + self.R1)) ** 2 - self.R1 ** 2)) / 2
            + 1j * self.W3 / 2.0
        )
        Z5 = Z6 + self.H3
        rot_sign = 1
    else:  # inward slot
        Z7 = Z8 - self.H0
        # Rotation to get the tooth on X axis
        Z7 = Z7 * exp(1j * slot_pitch / 2)
        Z8 = Z8 * exp(1j * slot_pitch / 2)
        Z6 = (
            Z7.real
            - sqrt(-4 * ((Z7.imag - (self.W3 / 2.0 + self.R1)) ** 2 - self.R1 ** 2)) / 2
            + 1j * self.W3 / 2.0
        )
        Z5 = Z6 - self.H3
        rot_sign = -1

    # Tooth ref to slot
    Z1, Z2, Z3, Z4 = (
        Z8 * exp(-1j * slot_pitch / 2),
        Z7 * exp(-1j * slot_pitch / 2),
        Z6 * exp(-1j * slot_pitch / 2),
        Z5 * exp(-1j * slot_pitch / 2),
    )

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    # symetry
    point_dict["Z5"] = Z4.conjugate()
    point_dict["Z6"] = Z3.conjugate()
    point_dict["Z7"] = Z2.conjugate()
    point_dict["Z8"] = Z1.conjugate()
    # Center
    A = Arc1(Z2, Z3, rot_sign * self.R1, self.is_outwards())
    point_dict["Zc1"] = A.get_center()
    point_dict["Zc2"] = (point_dict["Z4"] + point_dict["Z5"]) / 2
    point_dict["Zc3"] = point_dict["Zc1"].conjugate()
    return point_dict



from numpy import arcsin, exp

from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def get_surface_active(self, alpha=0, delta=0):
    """Return the full winding surface

    Parameters
    ----------
    self : SlotUD
        A SlotUD object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_wind: Surface
        Surface corresponding to the Winding Area
    """
    # get the name of the lamination
    st = self.get_name_lam()

    line_list = self.build_geometry()[self.wind_begin_index : self.wind_end_index]
    if self.type_line_wind == 0:
        line_list.append(
            Segment(begin=line_list[-1].get_end(), end=line_list[0].get_begin())
        )
    else:
        line_list.append(
            Arc1(
                begin=line_list[-1].get_end(),
                end=line_list[0].get_begin(),
                radius=-abs(line_list[-1].get_end()),
                is_trigo_direction=False,
            )
        )

    surface = SurfLine(line_list=line_list, label="Wind_" + st + "_R0_T0_S0")
    surface.comp_point_ref(is_set=True)

    # Apply transformation
    surface.rotate(alpha)
    surface.translate(delta)

    return surface



# -*- coding: utf-8 -*-

import numpy as np


def get_ref_point(self, vertice, point):
    """Return the coordinate of the equivalent point in the ref cell

    Parameters
    ----------
    self : RefSegmentP1
        a RefSegmentP1 object
    vertice : ndarray
        vertice of the cell
    point : ndarray
        coordinates of a point

        Returns
    -------
    pt1_ref : ndarray
        coordinates of the ref point
    """

    point_ref = np.array([0, 0], dtype=float)
    point_decal = np.array([-1, 0], dtype=float)

    pt1 = point[0:2] - vertice[0, :]
    pt2 = vertice[1, :] - vertice[0, :]
    rho2 = np.sqrt(pt2[0] ** 2 + pt2[1] ** 2)
    phi2 = np.arctan2(pt2[1], pt2[0])
    Trot = np.array([[np.cos(phi2), np.sin(phi2)], [-np.sin(phi2), np.cos(phi2)]])

    pt1_ref = 2 * np.dot(Trot, pt1) / rho2 + point_decal
    # pt2_ref =np.dot(Trot, pt2) / rho2 - point_decal

    return pt1_ref



# -*- coding: utf-8 -*-

from numpy import cos


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotM10
        A SlotM10 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]

    Rbo = self.get_Rbo()

    if self.is_outwards():
        return abs(Z2) - Rbo
    else:
        return Rbo - abs((Z2 + Z3) / 2)



# -*- coding: utf-8 -*-

from ....Classes.OutElec import OutElec
from ....Classes.Simulation import Simulation
from ....Methods.Simulation.Input import InputError


def gen_input(self):
    """Generate the input for the magnetic module (electrical output)

    Parameters
    ----------
    self : InputCurrent
        An InputCurrent object
    """

    output = OutElec()

    # get the simulation
    if isinstance(self.parent, Simulation):
        simu = self.parent
    elif isinstance(self.parent.parent, Simulation):
        simu = self.parent.parent
    else:
        raise InputError(
            "ERROR: InputCurrent object should be inside a Simulation object"
        )

    output.N0 = self.N0
    output.felec = self.comp_felec()

    # Set discretization
    Time, Angle = self.comp_axes(simu.machine, self.N0)
    output.Time = Time
    output.Angle = Angle

    # Initialize output at None
    output.Id_ref = None
    output.Iq_ref = None
    output.Ud_ref = None
    output.Uq_ref = None
    output.Is = None
    output.Ir = None

    # Load and check voltage and currents
    if self.Ud_ref is not None and self.Uq_ref is not None:
        output.Ud_ref = self.Ud_ref
        output.Uq_ref = self.Uq_ref
        simu.elec.eec.parameters["Ud"] = self.Ud_ref
        simu.elec.eec.parameters["Uq"] = self.Uq_ref
        if self.Id_ref is not None and self.Iq_ref is not None:
            output.Id_ref = self.Id_ref
            output.Iq_ref = self.Iq_ref
        else:
            output.Id_ref = 1
            output.Iq_ref = 1
    elif self.Id_ref is not None and self.Iq_ref is not None:
        output.Id_ref = self.Id_ref
        output.Iq_ref = self.Iq_ref
    else:
        raise InputError("ERROR: Id/Iq or Ud/Uq missing")

    # Load and check rot_dir
    if self.rot_dir is None or self.rot_dir not in [-1, 1]:
        # Enforce default rotation direction
        simu.parent.geo.rot_dir = -1
    else:
        simu.parent.geo.rot_dir = self.rot_dir

    if simu.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    # Save the Output in the correct place
    simu.parent.elec = output



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW22
        A SlotW22 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """
    Rbo = self.get_Rbo()

    Swind = self.comp_surface_active()

    # Computation of isthmus surface
    if self.is_outwards():
        # Surface of the external disk
        Sext = ((self.H0 + Rbo) ** 2) * pi

        # Surface of the internal disk
        Sint = (Rbo ** 2) * pi
    else:
        # Surface of the external disk
        Sext = (Rbo ** 2) * pi

        # Surface of the internal disk
        Sint = ((Rbo - self.H0) ** 2) * pi
    # Surface of the ring
    Sring = Sext - Sint

    # Only an W0 angle of the ring
    Sisth = Sring * self.W0 / (2 * pi)

    return Swind + Sisth



# -*- coding: utf-8 -*-

from numpy import arcsin, exp

from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve
    in the list

    Parameters
    ----------
    self : SlotW12
        A SlotW12 object

    Returns
    -------
    curve_list: list
        A list of 4 Segment and 3 Arc3

    """

    if self.is_outwards():
        rot_sign = True
    else:  # inward slot
        rot_sign = False

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    if self.R1 > 0:  # R1=0 => Z2==Z3
        curve_list.append(Arc3(Z2, Z3, rot_sign))
    if self.H1 > 0:  # H1=0 => Z3==Z4
        curve_list.append(Segment(Z3, Z4))
    curve_list.append(Arc3(Z4, Z5, rot_sign))
    if self.H1 > 0:  # H1=0 => Z5==Z6
        curve_list.append(Segment(Z5, Z6))
    if self.R1 > 0:  # R1=0 => Z6==Z7
        curve_list.append(Arc3(Z6, Z7, rot_sign))
    curve_list.append(Segment(Z7, Z8))

    return curve_list



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the Trapeze surface

    Parameters
    ----------
    self : Trapeze
        A Trapeze object

    Returns
    -------
    surf: float
        The Trapeze surface [m**2]

    """

    return self.height * (self.W2 + self.W1) / 2



# -*- coding: utf-8 -*-
from numpy import exp, pi

from ...Classes.Arc1 import Arc1
from ...Classes.Arc2 import Arc2
from ...Classes.Circle import Circle
from ...Classes.Segment import Segment
from ...Classes.SurfLine import SurfLine


def get_air_box(sym, machine):
    """Returns  an outer surface surrounding the external lamination

    Parameters
    ----------
    sym: int
        Symmetry factor (1= full machine, 2= half of the machine...)
    machine: float
        to extract laminations

    Returns
    -------
    surf_list: list
        Outer surface (Air Box)
    """
    lam_list = machine.get_lam_list()
    lam_int = lam_list[0]
    lam_ext = lam_list[1]

    R_ext = lam_ext.get_Ryoke()
    R_ab = 1.5 * R_ext  # Default at 1.5 times external lam radius

    surf_list = list()
    if sym == 1:  # Complete machine
        # TO-DO: Airbox Full Machine no implemented yet.

        surf_list.append(
            Circle(
                center=0,
                radius=R_ab,
                label="Airbox",
                point_ref=(R_ab / 2) * exp(1j * pi / 2),
                line_label="Airbox_arc",
            )
        )
    else:  # Symmetry
        # Internal AirGap
        Z1 = R_ext
        Z0 = Z1 * exp(1j * 2 * pi / sym)
        Z2 = R_ab
        Z3 = Z2 * exp(1j * 2 * pi / sym)
        airbox_lines = list()
        airbox_lines.append(Segment(begin=Z1, end=Z2, label="airbox_line_1"))
        airbox_lines.append(
            Arc1(
                begin=Z2,
                end=Z3,
                radius=R_ab,
                label="airbox_arc",
                is_trigo_direction=True,
            )
        )
        airbox_lines.append(Segment(begin=Z3, end=Z0, label="airbox_line_2"))
        airbox_lines.append(
            Arc2(begin=Z0, center=0.0, angle=-2 * pi / sym, label="lam_ext_arc")
        )
        surf_list.append(
            SurfLine(
                line_list=airbox_lines,
                # point_ref=0.0 * Z2 * exp(1j * pi / sym),
                label="Airbox",
            )
        )

    return surf_list



# -*- coding: utf-8 -*-


def comp_surface_magnet_id(self, index):
    """Compute the surface of the hole magnet of the corresponding index

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object
    index : int
        Index of the magnet to compute the surface

    Returns
    -------
    Smag: float
        Surface of the Magnet [m**2]
    """

    if index == 0 and self.magnet_0:
        return self.W0 * self.H1
    return 0



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/Machine.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/Machine
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.Machine.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Machine.Machine.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Machine.Machine.comp_angle_offset_initial import (
        comp_angle_offset_initial,
    )
except ImportError as error:
    comp_angle_offset_initial = error

try:
    from ..Methods.Machine.Machine.comp_desc_dict import comp_desc_dict
except ImportError as error:
    comp_desc_dict = error

try:
    from ..Methods.Machine.Machine.comp_length_airgap_active import (
        comp_length_airgap_active,
    )
except ImportError as error:
    comp_length_airgap_active = error

try:
    from ..Methods.Machine.Machine.comp_masses import comp_masses
except ImportError as error:
    comp_masses = error

try:
    from ..Methods.Machine.Machine.comp_output_geo import comp_output_geo
except ImportError as error:
    comp_output_geo = error

try:
    from ..Methods.Machine.Machine.comp_Rgap_mec import comp_Rgap_mec
except ImportError as error:
    comp_Rgap_mec = error

try:
    from ..Methods.Machine.Machine.comp_periodicity import comp_periodicity
except ImportError as error:
    comp_periodicity = error

try:
    from ..Methods.Machine.Machine.comp_width_airgap_mag import comp_width_airgap_mag
except ImportError as error:
    comp_width_airgap_mag = error

try:
    from ..Methods.Machine.Machine.comp_width_airgap_mec import comp_width_airgap_mec
except ImportError as error:
    comp_width_airgap_mec = error

try:
    from ..Methods.Machine.Machine.get_material_list import get_material_list
except ImportError as error:
    get_material_list = error

try:
    from ..Methods.Machine.Machine.get_polar_eq import get_polar_eq
except ImportError as error:
    get_polar_eq = error

try:
    from ..Methods.Machine.Machine.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Machine.Machine.plot_anim_rotor import plot_anim_rotor
except ImportError as error:
    plot_anim_rotor = error

try:
    from ..Methods.Machine.Machine.get_lam_list import get_lam_list
except ImportError as error:
    get_lam_list = error

try:
    from ..Methods.Machine.Machine.get_lam_list_label import get_lam_list_label
except ImportError as error:
    get_lam_list_label = error

try:
    from ..Methods.Machine.Machine.get_lam_by_label import get_lam_by_label
except ImportError as error:
    get_lam_by_label = error

try:
    from ..Methods.Machine.Machine.get_lam_index import get_lam_index
except ImportError as error:
    get_lam_index = error

try:
    from ..Methods.Machine.Machine.get_pole_pair_number import get_pole_pair_number
except ImportError as error:
    get_pole_pair_number = error


from ._check import InitUnKnowClassError
from .Frame import Frame
from .Shaft import Shaft


class Machine(FrozenClass):
    """Abstract class for machines"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.Machine.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Machine.Machine.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use Machine method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Machine.Machine.comp_angle_offset_initial
    if isinstance(comp_angle_offset_initial, ImportError):
        comp_angle_offset_initial = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_angle_offset_initial: "
                    + str(comp_angle_offset_initial)
                )
            )
        )
    else:
        comp_angle_offset_initial = comp_angle_offset_initial
    # cf Methods.Machine.Machine.comp_desc_dict
    if isinstance(comp_desc_dict, ImportError):
        comp_desc_dict = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_desc_dict: " + str(comp_desc_dict)
                )
            )
        )
    else:
        comp_desc_dict = comp_desc_dict
    # cf Methods.Machine.Machine.comp_length_airgap_active
    if isinstance(comp_length_airgap_active, ImportError):
        comp_length_airgap_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_length_airgap_active: "
                    + str(comp_length_airgap_active)
                )
            )
        )
    else:
        comp_length_airgap_active = comp_length_airgap_active
    # cf Methods.Machine.Machine.comp_masses
    if isinstance(comp_masses, ImportError):
        comp_masses = property(
            fget=lambda x: raise_(
                ImportError("Can't use Machine method comp_masses: " + str(comp_masses))
            )
        )
    else:
        comp_masses = comp_masses
    # cf Methods.Machine.Machine.comp_output_geo
    if isinstance(comp_output_geo, ImportError):
        comp_output_geo = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_output_geo: " + str(comp_output_geo)
                )
            )
        )
    else:
        comp_output_geo = comp_output_geo
    # cf Methods.Machine.Machine.comp_Rgap_mec
    if isinstance(comp_Rgap_mec, ImportError):
        comp_Rgap_mec = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_Rgap_mec: " + str(comp_Rgap_mec)
                )
            )
        )
    else:
        comp_Rgap_mec = comp_Rgap_mec
    # cf Methods.Machine.Machine.comp_periodicity
    if isinstance(comp_periodicity, ImportError):
        comp_periodicity = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_periodicity: "
                    + str(comp_periodicity)
                )
            )
        )
    else:
        comp_periodicity = comp_periodicity
    # cf Methods.Machine.Machine.comp_width_airgap_mag
    if isinstance(comp_width_airgap_mag, ImportError):
        comp_width_airgap_mag = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_width_airgap_mag: "
                    + str(comp_width_airgap_mag)
                )
            )
        )
    else:
        comp_width_airgap_mag = comp_width_airgap_mag
    # cf Methods.Machine.Machine.comp_width_airgap_mec
    if isinstance(comp_width_airgap_mec, ImportError):
        comp_width_airgap_mec = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method comp_width_airgap_mec: "
                    + str(comp_width_airgap_mec)
                )
            )
        )
    else:
        comp_width_airgap_mec = comp_width_airgap_mec
    # cf Methods.Machine.Machine.get_material_list
    if isinstance(get_material_list, ImportError):
        get_material_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_material_list: "
                    + str(get_material_list)
                )
            )
        )
    else:
        get_material_list = get_material_list
    # cf Methods.Machine.Machine.get_polar_eq
    if isinstance(get_polar_eq, ImportError):
        get_polar_eq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_polar_eq: " + str(get_polar_eq)
                )
            )
        )
    else:
        get_polar_eq = get_polar_eq
    # cf Methods.Machine.Machine.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use Machine method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Machine.Machine.plot_anim_rotor
    if isinstance(plot_anim_rotor, ImportError):
        plot_anim_rotor = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method plot_anim_rotor: " + str(plot_anim_rotor)
                )
            )
        )
    else:
        plot_anim_rotor = plot_anim_rotor
    # cf Methods.Machine.Machine.get_lam_list
    if isinstance(get_lam_list, ImportError):
        get_lam_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_lam_list: " + str(get_lam_list)
                )
            )
        )
    else:
        get_lam_list = get_lam_list
    # cf Methods.Machine.Machine.get_lam_list_label
    if isinstance(get_lam_list_label, ImportError):
        get_lam_list_label = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_lam_list_label: "
                    + str(get_lam_list_label)
                )
            )
        )
    else:
        get_lam_list_label = get_lam_list_label
    # cf Methods.Machine.Machine.get_lam_by_label
    if isinstance(get_lam_by_label, ImportError):
        get_lam_by_label = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_lam_by_label: "
                    + str(get_lam_by_label)
                )
            )
        )
    else:
        get_lam_by_label = get_lam_by_label
    # cf Methods.Machine.Machine.get_lam_index
    if isinstance(get_lam_index, ImportError):
        get_lam_index = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_lam_index: " + str(get_lam_index)
                )
            )
        )
    else:
        get_lam_index = get_lam_index
    # cf Methods.Machine.Machine.get_pole_pair_number
    if isinstance(get_pole_pair_number, ImportError):
        get_pole_pair_number = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Machine method get_pole_pair_number: "
                    + str(get_pole_pair_number)
                )
            )
        )
    else:
        get_pole_pair_number = get_pole_pair_number
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        frame=-1,
        shaft=-1,
        name="default_machine",
        desc="",
        type_machine=1,
        logger_name="Pyleecan.Machine",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "frame" in list(init_dict.keys()):
                frame = init_dict["frame"]
            if "shaft" in list(init_dict.keys()):
                shaft = init_dict["shaft"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "type_machine" in list(init_dict.keys()):
                type_machine = init_dict["type_machine"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.frame = frame
        self.shaft = shaft
        self.name = name
        self.desc = desc
        self.type_machine = type_machine
        self.logger_name = logger_name

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Machine_str = ""
        if self.parent is None:
            Machine_str += "parent = None " + linesep
        else:
            Machine_str += "parent = " + str(type(self.parent)) + " object" + linesep
        if self.frame is not None:
            tmp = self.frame.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Machine_str += "frame = " + tmp
        else:
            Machine_str += "frame = None" + linesep + linesep
        if self.shaft is not None:
            tmp = self.shaft.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Machine_str += "shaft = " + tmp
        else:
            Machine_str += "shaft = None" + linesep + linesep
        Machine_str += 'name = "' + str(self.name) + '"' + linesep
        Machine_str += 'desc = "' + str(self.desc) + '"' + linesep
        Machine_str += "type_machine = " + str(self.type_machine) + linesep
        Machine_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        return Machine_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.frame != self.frame:
            return False
        if other.shaft != self.shaft:
            return False
        if other.name != self.name:
            return False
        if other.desc != self.desc:
            return False
        if other.type_machine != self.type_machine:
            return False
        if other.logger_name != self.logger_name:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.frame is None and self.frame is not None) or (
            other.frame is not None and self.frame is None
        ):
            diff_list.append(name + ".frame None mismatch")
        elif self.frame is not None:
            diff_list.extend(self.frame.compare(other.frame, name=name + ".frame"))
        if (other.shaft is None and self.shaft is not None) or (
            other.shaft is not None and self.shaft is None
        ):
            diff_list.append(name + ".shaft None mismatch")
        elif self.shaft is not None:
            diff_list.extend(self.shaft.compare(other.shaft, name=name + ".shaft"))
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._desc != self._desc:
            diff_list.append(name + ".desc")
        if other._type_machine != self._type_machine:
            diff_list.append(name + ".type_machine")
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.frame)
        S += getsizeof(self.shaft)
        S += getsizeof(self.name)
        S += getsizeof(self.desc)
        S += getsizeof(self.type_machine)
        S += getsizeof(self.logger_name)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Machine_dict = dict()
        if self.frame is None:
            Machine_dict["frame"] = None
        else:
            Machine_dict["frame"] = self.frame.as_dict(**kwargs)
        if self.shaft is None:
            Machine_dict["shaft"] = None
        else:
            Machine_dict["shaft"] = self.shaft.as_dict(**kwargs)
        Machine_dict["name"] = self.name
        Machine_dict["desc"] = self.desc
        Machine_dict["type_machine"] = self.type_machine
        Machine_dict["logger_name"] = self.logger_name
        # The class name is added to the dict for deserialisation purpose
        Machine_dict["__class__"] = "Machine"
        return Machine_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.frame is not None:
            self.frame._set_None()
        if self.shaft is not None:
            self.shaft._set_None()
        self.name = None
        self.desc = None
        self.type_machine = None
        self.logger_name = None

    def _get_frame(self):
        """getter of frame"""
        return self._frame

    def _set_frame(self, value):
        """setter of frame"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "frame"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Frame()
        check_var("frame", value, "Frame")
        self._frame = value

        if self._frame is not None:
            self._frame.parent = self

    frame = property(
        fget=_get_frame,
        fset=_set_frame,
        doc=u"""Machine's Frame

        :Type: Frame
        """,
    )

    def _get_shaft(self):
        """getter of shaft"""
        return self._shaft

    def _set_shaft(self, value):
        """setter of shaft"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "shaft"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Shaft()
        check_var("shaft", value, "Shaft")
        self._shaft = value

        if self._shaft is not None:
            self._shaft.parent = self

    shaft = property(
        fget=_get_shaft,
        fset=_set_shaft,
        doc=u"""Machine's Shaft

        :Type: Shaft
        """,
    )

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the machine

        :Type: str
        """,
    )

    def _get_desc(self):
        """getter of desc"""
        return self._desc

    def _set_desc(self, value):
        """setter of desc"""
        check_var("desc", value, "str")
        self._desc = value

    desc = property(
        fget=_get_desc,
        fset=_set_desc,
        doc=u"""Machine description

        :Type: str
        """,
    )

    def _get_type_machine(self):
        """getter of type_machine"""
        return self._type_machine

    def _set_type_machine(self, value):
        """setter of type_machine"""
        check_var("type_machine", value, "int")
        self._type_machine = value

    type_machine = property(
        fget=_get_type_machine,
        fset=_set_type_machine,
        doc=u"""Integer to store the machine type (for the GUI, should be replaced by a test of the object type)

        :Type: int
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError


def check(self):
    """Check that the SlotM17 object is correct

    Parameters
    ----------
    self : SlotM17
        A SlotM17 object

    Returns
    -------
    None
    """
    if self.parent is None:
        raise SlotCheckError("SlotM17 must be inside a lamination")
    if self.Zs != 2:
        raise SlotCheckError("SlotM17 must have Zs=2")
    if not self.parent.is_internal or self.parent.Rint != 0:
        raise SlotCheckError(
            "SlotM17 must be inside an internal lamination with Rint=0"
        )



# -*- coding: utf-8 -*-

from ....Functions.Structural.conversions import cart2pol


def get_shape_pol(self):
    """Return the mode shape in cartesian coordinates.

    Parameters
    ----------
    self : Mode
        a Mode object

    Returns
    -------
    shape : ndarray
        ndarray of the shape (Nnodes*Ndof)
    """

    if self.shape_pol is not None and self.shape_pol.size != 0:
        return self.shape_pol
    else:
        shape_xyz = self.shape_xyz
        points = self.parent.mesh.get_points()
        return cart2pol(shape_xyz, points)



# -*- coding: utf-8 -*-


def translate(self, Zt):
    """Translate the surface

    Parameters
    ----------
    self : SurfLine
        A SurfLine object

    Zt : complex
        Complex value for translation

    Returns
    -------
    None
    """
    # Check if the Surface is correct
    self.check()

    # Translation  of every line in the Surface
    for line in self.line_list:
        line.translate(Zt)

    if self.point_ref is not None:
        self.point_ref += Zt



# -*- coding: utf-8 -*-

from .....Methods.Output.Output.getter import GetOutError
from numpy import pi, cumsum, roll, size, ones


def get_angle_rotor(self):
    """
    Return the angular position of the rotor as a function of time
    and set the Output.elec.angle_rotor attribute if it is None

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    alpha_rotor: numpy.ndarray
        angular position of the rotor as a function of time (vector) [rad]

    """

    # Already available => Return
    if self.elec.angle_rotor is not None and self.elec.angle_rotor.size > 0:
        return self.elec.angle_rotor
    else:  # Compute according to the speed
        Nr = self.elec.get_Nr()

        # Get rotor rotating direction
        rot_dir = (
            -self.get_rot_dir()
        )  # rotor rotating is the opposite of rot_dir which is fundamental field rotation direction so that rotor moves in positive angles

        # Compute rotor initial angle (for synchronous machines, to align rotor d-axis and stator alpha-axis)
        A0 = self.get_angle_offset_initial()

        time = self.elec.Time.get_values(is_oneperiod=False)
        if time.size == 1:
            # Only one time step, no need to compute the position
            return ones(1) * A0
        else:
            deltaT = time[1] - time[0]
            # Convert Nr from [rpm] to [rad/s] (time in [s] and angle_rotor in [rad])
            Ar = cumsum(rot_dir * deltaT * Nr * 2 * pi / 60)
            # Enforce first position to 0
            Ar = roll(Ar, 1)
            Ar[0] = 0
            self.elec.angle_rotor = Ar + A0
            return self.elec.angle_rotor



# -*- coding: utf-8 -*-

from SciDataTool import VectorField


def get_data(self):
    """Generate VectorField object

    Parameters
    ----------
    self : ImportVectorField
        An ImportVectorField object

    Returns
    -------
    VectField: VectorField
        The generated VectorField object

    """

    comp_dict = {}
    for key, component in self.components.items():
        comp_dict[key] = component.get_data()

    VectField = VectorField(name=self.name, symbol=self.symbol, components=comp_dict)

    return VectField



# -*- coding: utf-8 -*-
from ....Methods.Simulation.Input import InputError
from ....Classes.OutStruct import OutStruct


def comp_axes(self, output):
    """Compute axes used for the Structural module

    Parameters
    ----------
    self : Structural
        a Structural object
    output : Output
        an Output object (to update)
    """
    if self.parent is None:
        raise InputError(
            "ERROR: The Structural object must be in a Simulation object to run"
        )
    if self.parent.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    # setup OutStruct if None
    if self.parent.parent.struct is None:
        self.parent.parent.struct = OutStruct()

    # readability
    N0 = getattr(self.parent.input, "N0", None)
    machine = self.parent.machine

    Time, Angle = self.parent.input.comp_axes(machine, N0=N0)

    # TODO maybe remove periodicity ?

    output.struct.Time = Time
    # output.struct.Nt_tot = len(output.struct.time)

    output.struct.Angle = Angle
    # output.struct.Na_tot = len(output.struct.angle)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Optimization/OptiDesignVar.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Optimization/OptiDesignVar
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ParamExplorer import ParamExplorer

from ntpath import basename
from os.path import isfile
from ._check import CheckTypeError
import numpy as np
import random
from ._check import InitUnKnowClassError


class OptiDesignVar(ParamExplorer):
    """Optimization"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        type_var="interval",
        space=[0, 1],
        get_value=None,
        name="",
        symbol="",
        unit="",
        setter=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "type_var" in list(init_dict.keys()):
                type_var = init_dict["type_var"]
            if "space" in list(init_dict.keys()):
                space = init_dict["space"]
            if "get_value" in list(init_dict.keys()):
                get_value = init_dict["get_value"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "setter" in list(init_dict.keys()):
                setter = init_dict["setter"]
        # Set the properties (value check and convertion are done in setter)
        self.type_var = type_var
        self.space = space
        self.get_value = get_value
        # Call ParamExplorer init
        super(OptiDesignVar, self).__init__(
            name=name, symbol=symbol, unit=unit, setter=setter
        )
        # The class is frozen (in ParamExplorer init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        OptiDesignVar_str = ""
        # Get the properties inherited from ParamExplorer
        OptiDesignVar_str += super(OptiDesignVar, self).__str__()
        OptiDesignVar_str += 'type_var = "' + str(self.type_var) + '"' + linesep
        OptiDesignVar_str += (
            "space = "
            + linesep
            + str(self.space).replace(linesep, linesep + "\t")
            + linesep
        )
        if self._get_value_str is not None:
            OptiDesignVar_str += "get_value = " + self._get_value_str + linesep
        elif self._get_value_func is not None:
            OptiDesignVar_str += "get_value = " + str(self._get_value_func) + linesep
        else:
            OptiDesignVar_str += "get_value = None" + linesep + linesep
        return OptiDesignVar_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ParamExplorer
        if not super(OptiDesignVar, self).__eq__(other):
            return False
        if other.type_var != self.type_var:
            return False
        if other.space != self.space:
            return False
        if other._get_value_str != self._get_value_str:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ParamExplorer
        diff_list.extend(super(OptiDesignVar, self).compare(other, name=name))
        if other._type_var != self._type_var:
            diff_list.append(name + ".type_var")
        if other._space != self._space:
            diff_list.append(name + ".space")
        if other._get_value_str != self._get_value_str:
            diff_list.append(name + ".get_value")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ParamExplorer
        S += super(OptiDesignVar, self).__sizeof__()
        S += getsizeof(self.type_var)
        if self.space is not None:
            for value in self.space:
                S += getsizeof(value)
        S += getsizeof(self._get_value_str)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from ParamExplorer
        OptiDesignVar_dict = super(OptiDesignVar, self).as_dict(**kwargs)
        OptiDesignVar_dict["type_var"] = self.type_var
        OptiDesignVar_dict["space"] = (
            self.space.copy() if self.space is not None else None
        )
        if self._get_value_str is not None:
            OptiDesignVar_dict["get_value"] = self._get_value_str
        elif "keep_function" in kwargs and kwargs["keep_function"]:
            OptiDesignVar_dict["get_value"] = self.get_value
        else:
            OptiDesignVar_dict["get_value"] = None
            if self.get_value is not None:
                self.get_logger().warning(
                    "OptiDesignVar.as_dict(): "
                    + f"Function {self.get_value.__name__} is not serializable "
                    + "and will be converted to None."
                )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        OptiDesignVar_dict["__class__"] = "OptiDesignVar"
        return OptiDesignVar_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.type_var = None
        self.space = None
        self.get_value = None
        # Set to None the properties inherited from ParamExplorer
        super(OptiDesignVar, self)._set_None()

    def _get_type_var(self):
        """getter of type_var"""
        return self._type_var

    def _set_type_var(self, value):
        """setter of type_var"""
        check_var("type_var", value, "str")
        self._type_var = value

    type_var = property(
        fget=_get_type_var,
        fset=_set_type_var,
        doc="""Type of the variable interval or set.

        :Type: str
        """,
    )

    def _get_space(self):
        """getter of space"""
        return self._space

    def _set_space(self, value):
        """setter of space"""
        if type(value) is int and value == -1:
            value = list()
        check_var("space", value, "list")
        self._space = value

    space = property(
        fget=_get_space,
        fset=_set_space,
        doc="""Space of the variable

        :Type: list
        """,
    )

    def _get_get_value(self):
        """getter of get_value"""
        return self._get_value_func

    def _set_get_value(self, value):
        """setter of get_value"""
        if value is None:
            self._get_value_str = None
            self._get_value_func = None
        elif isinstance(value, str) and "lambda" in value:
            self._get_value_str = value
            self._get_value_func = eval(value)
        elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
            self._get_value_str = value
            f = open(value, "r")
            exec(f.read(), globals())
            self._get_value_func = eval(basename(value[:-3]))
        elif callable(value):
            self._get_value_str = None
            self._get_value_func = value
        else:
            raise CheckTypeError(
                "For property get_value Expected function or str (path to python file or lambda), got: "
                + str(type(value))
            )

    get_value = property(
        fget=_get_get_value,
        fset=_set_get_value,
        doc="""Function of the space to initiate the variable

        :Type: function
        """,
    )



from numpy import (
    array,
    matmul,
    sqrt,
    cos,
    sin,
    reshape,
    newaxis,
    finfo,
    log10,
    floor,
    pi,
    linspace,
    column_stack,
    vstack,
)

EPS = int(floor(-log10(finfo(float).eps)))

# TODO: add homopolar component


def ab2n(Z_ab, n=3, rot_dir=-1):
    """
    2 phase equivalent to n phase coordinate transformation, i.e. Clarke transformation

    Parameters
    ----------
    Z_ab : ndarray
        matrix (N x 2) of 2 phase equivalent values
    n : integer
        number of phases
    rot_dir : integer
        rotation direction of the fundamental of magnetic field (rot_dir = +/- 1)

    Returns
    -------
    Z_n : ndarray
        transformed matrix (N x n) of n phase values

    """
    ii = linspace(0, n - 1, n)
    alpha = (
        rot_dir * 2 * ii * pi / n
    )  # Phasor depending on fundamental field rotation direction

    # Transformation matrix
    ab_2_n = vstack((cos(alpha).round(decimals=EPS), -sin(alpha).round(decimals=EPS)))

    Z_n = matmul(Z_ab, ab_2_n)

    return Z_n


def n2ab(Z_n, n=3, rot_dir=-1):
    """n phase to 2 phase equivalent coordinate transformation, i.e. Clarke transformation

    Parameters
    ----------
    Z_n : ndarray
        matrix (N x n) of n phase values
    n : integer
        number of phases
    rot_dir : integer
        rotation direction of the fundamental of magnetic field (rot_dir = +/- 1)

    Returns
    -------
    Z_ab : ndarray
        transformed matrix (N x 2) of 2 phase equivalent values

    """
    ii = linspace(0, n - 1, n)
    alpha = (
        rot_dir * 2 * ii * pi / n
    )  # Phasor depending on fundamental field rotation direction

    # Transformation matrix
    n_2_ab = (
        2
        / n
        * column_stack(
            (cos(alpha).round(decimals=EPS), -sin(alpha).round(decimals=EPS))
        )
    )

    Z_ab = matmul(Z_n, n_2_ab)

    return Z_ab


def ab2dq(Z_ab, theta):
    """
    alpha-beta to dq coordinate transformation
    NOTE: sin/cos values are rounded to avoid numerical errors

    Parameters
    ----------
    Z_ab : ndarray
        matrix (N x 2) of alpha-beta - reference frame values
    theta : ndarray
        angle of the rotor coordinate system

    Returns
    -------
    Z_dq : ndarray
        transformed (dq) values

    """
    if len(Z_ab.shape) == 1:
        Z_ab = Z_ab[newaxis, :]

    sin_theta = sin(theta).round(decimals=EPS)
    cos_theta = cos(theta).round(decimals=EPS)

    Z_d = Z_ab[:, 0] * cos_theta + Z_ab[:, 1] * sin_theta
    Z_q = -Z_ab[:, 0] * sin_theta + Z_ab[:, 1] * cos_theta

    return reshape([Z_d, Z_q], (2, -1)).transpose()


def dq2ab(Z_dq, theta):
    """
    dq to alpha-beta coordinate transformation
    NOTE: sin/cos values are rounded to avoid numerical errors

    Parameters
    ----------
    Z_dq : ndarray
        matrix (N x 2) of dq - reference frame values
    theta : ndarray
        angle of the rotor coordinate system

    Returns
    -------
    Z_ab : ndarray
        transformed array

    """
    if len(Z_dq.shape) == 1:
        Z_dq = Z_dq[newaxis, :]

    sin_theta = sin(theta).round(decimals=EPS)
    cos_theta = cos(theta).round(decimals=EPS)

    # Multiply by sqrt(2) to go from (Id_rms, Iq_rms) in to I_ab in amplitude
    Z_a = Z_dq[:, 0] * cos_theta - Z_dq[:, 1] * sin_theta
    Z_b = Z_dq[:, 0] * sin_theta + Z_dq[:, 1] * cos_theta

    return reshape([Z_a, Z_b], (2, -1)).transpose()


def n2dq(Z_n, theta, n=3, rot_dir=-1, is_dq_rms=True):
    """n phase to dq equivalent coordinate transformation

    Parameters
    ----------
    Z_n : ndarray
        matrix (N x n) of n phase values
    n : integer
        number of phases
    rot_dir : integer
        rotation direction of the fundamental of magnetic field (rot_dir = +/- 1)
    is_dq_rms : boolean
        True to return dq currents in rms value (Pyleecan convention), False to return peak values

    Returns
    -------
    Z_dq : ndarray
        transformed matrix (N x 2) of dq equivalent values

    """

    Z_dq = ab2dq(n2ab(Z_n, n=n, rot_dir=rot_dir), theta)

    if is_dq_rms == True:
        # Divide by sqrt(2) to go from (Id_peak, Iq_peak) to (Id_rms, Iq_rms)
        Z_dq = Z_dq / sqrt(2)

    return Z_dq


def dq2n(Z_dq, theta, n=3, rot_dir=-1, is_n_rms=False):
    """n phase to dq equivalent coordinate transformation

    Parameters
    ----------
    Z_dq : ndarray
        matrix (N x 2) of dq phase values
    n : integer
        number of phases
    rot_dir : integer
        rotation direction of the fundamental of magnetic field (rot_dir = +/- 1)
    is_n_rms : boolean
        True to return n currents in rms value, False to return peak values (Pyleecan convention)

    Returns
    -------
    Z_n : ndarray
        transformed matrix (N x n) of n phase values

    """

    Z_n = ab2n(dq2ab(Z_dq, theta), n=n, rot_dir=rot_dir)

    if is_n_rms == False:
        # Multiply by sqrt(2) to from (I_n_rms) to (I_n_peak)
        Z_n = Z_n * sqrt(2)

    return Z_n



# -*- coding: utf-8 -*-
def check(self):
    """assert the Surface is correct

    Parameters
    ----------
    self : SurfRing
        A SurfRing object

    Returns
    -------
    None
    """
    self.out_surf.check()
    self.in_surf.check()



# -*- coding: utf-8 -*-

from numpy import angle as np_angle, exp, linspace

from ....Methods.Machine import ARC_NPOINT_D
from ....Methods.Geometry.Arc1 import NbPointArc1DError


def discretize(self, nb_point=ARC_NPOINT_D):
    """Return the discretize version of the Arc.
    Begin and end are always returned

    Parameters
    ----------
    self : Arc1
        The Arc1 object to discretize
    nb_point :
        Number of points to add to discretize the arc (Default value = ARC_NPOINT_D)

    Returns
    -------
    list_point: list
        list of complex coordinate of the points

    Raises
    ------
    NbPointArc1DError
        nb_point must be an integer >=0
    """

    # Check that the Arc is correct
    self.check()
    if not isinstance(nb_point, int):
        raise NbPointArc1DError("discretize : nb_point must be an integer")
    if nb_point < 0:
        raise NbPointArc1DError("nb_point must be >=0")

    # We use the complex representation of the point
    z1 = self.begin
    z2 = self.end
    zc = self.get_center()

    # Geometric transformation : center is the origin, angle(begin) = 0
    Zstart = (z1 - zc) * exp(-1j * np_angle(z1 - zc))
    Zend = (z2 - zc) * exp(-1j * np_angle(z1 - zc))

    # Generation of the point by rotation
    if self.is_trigo_direction:
        sign = 1
    else:
        sign = -1
    t = linspace(0, self.get_angle(), nb_point + 2)
    list_point = Zstart * exp(1j * t)

    # Geometric transformation : return to the main axis
    list_point = list_point * exp(1j * np_angle(z1 - zc)) + zc

    return list_point



def __missing__(self, key):
    """
    Called by XOutput.__getitem__() to implement self[key]
    for dict subclasses when key is not in the dictionary.
    """
    return self.xoutput_dict.__missing__(key)
